using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using CADImport;
using CADImport.FaceModule;
using CADImport.CADImportForms;

namespace CADImportNetDemos.AddEntityDemo
{
	#region Help
	/// <summary>
	/// Represents a form in which the properties of the circle are set 
	/// before adding it to the current CAD image.
	/// </summary>
	#endregion Help
	public class AddCircleForm : System.Windows.Forms.Form
	{
		private CADImage cadImage;				
		private CADCircle circle;
		private ColorDialog setColorDlg;

		#region Help
		/// <summary>
		/// Invokes on changing of <see cref="CADImportNetDemos.AddEntityDemo.AddPolyForm.Image"/>image drawing area
		/// </summary>
		#endregion
		public event CADImportEventHandler ResizeImageEvent;		

		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Label lbColor;
		private System.Windows.Forms.TextBox tbColor;
		private System.Windows.Forms.Button btnPickColor;
		private System.Windows.Forms.TextBox tbX;
		private System.Windows.Forms.TextBox tbY;
		private System.Windows.Forms.TextBox tbZ;
		private System.Windows.Forms.Label lbX;
		private System.Windows.Forms.Label lbY;
		private System.Windows.Forms.Label lbZ;
		private System.Windows.Forms.TextBox tbRadius;
		private System.Windows.Forms.Label lbRad;
		private System.Windows.Forms.Label lbWeight;
		private System.Windows.Forms.ComboBox cbLineWeight;		
		private System.ComponentModel.Container components = null;
		
		#region Help
		/// <summary>
		/// Raises the <see cref="CADImportNetDemos.AddEntityDemo.AddCircleForm.ResizeImageEvent"/>ResizeImageEvent event. 
		/// </summary>
		#endregion
		public void OnResizeImage()
		{
			if (ResizeImageEvent != null)
				ResizeImageEvent();
		}

		#region Help
		/// <summary>
		/// Gets or sets <see cref="CADImport.CADImage">CADImage</see> object where new entities are added
		/// </summary>
		#endregion
		public CADImage Image
		{
			get
			{
				return this.cadImage;
			}
			set
			{
				this.cadImage = value;
			}
		}

		private void InitParams()
		{
			setColorDlg = new ColorDialog();
			cbLineWeight.SelectedItem = ApplicationConstants.onestr;
			tbColor.Enabled = false;
			StartPosition = FormStartPosition.CenterScreen;
		}
			
		#region Help
		/// <summary>
		/// Initializes a new instance of the <see cref="CADImportNetDemos.AddEntityDemo.AddCircleForm">AddCircleForm</see> class.
		/// </summary>
		#endregion Help
		public AddCircleForm()
		{
			InitializeComponent();
			InitParams();		
		}

		#region Help
		/// <summary>
		/// Cleans up any resources being used.
		/// </summary>
		/// <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
		///</param>
		#endregion Help
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.lbColor = new System.Windows.Forms.Label();
			this.tbColor = new System.Windows.Forms.TextBox();
			this.btnPickColor = new System.Windows.Forms.Button();
			this.tbX = new System.Windows.Forms.TextBox();
			this.tbY = new System.Windows.Forms.TextBox();
			this.tbZ = new System.Windows.Forms.TextBox();
			this.lbX = new System.Windows.Forms.Label();
			this.lbY = new System.Windows.Forms.Label();
			this.lbZ = new System.Windows.Forms.Label();
			this.tbRadius = new System.Windows.Forms.TextBox();
			this.lbRad = new System.Windows.Forms.Label();
			this.lbWeight = new System.Windows.Forms.Label();
			this.cbLineWeight = new System.Windows.Forms.ComboBox();
			this.SuspendLayout();
			// 
			// btnOK
			// 
			this.btnOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnOK.Location = new System.Drawing.Point(96, 216);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(75, 25);
			this.btnOK.TabIndex = 0;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.addCircleOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnCancel.Location = new System.Drawing.Point(185, 216);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(75, 25);
			this.btnCancel.TabIndex = 1;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.Click += new System.EventHandler(this.addCircleCancel_Click);
			// 
			// lbColor
			// 
			this.lbColor.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.lbColor.Location = new System.Drawing.Point(8, 16);
			this.lbColor.Name = "lbColor";
			this.lbColor.Size = new System.Drawing.Size(56, 23);
			this.lbColor.TabIndex = 2;
			this.lbColor.Text = "Color :";
			// 
			// tbColor
			// 
			this.tbColor.BackColor = System.Drawing.Color.Black;
			this.tbColor.Location = new System.Drawing.Point(96, 16);
			this.tbColor.Multiline = true;
			this.tbColor.Name = "tbColor";
			this.tbColor.Size = new System.Drawing.Size(75, 25);
			this.tbColor.TabIndex = 3;
			this.tbColor.Text = "";
			// 
			// btnPickColor
			// 
			this.btnPickColor.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnPickColor.Location = new System.Drawing.Point(185, 16);
			this.btnPickColor.Name = "btnPickColor";
			this.btnPickColor.Size = new System.Drawing.Size(75, 25);
			this.btnPickColor.TabIndex = 4;
			this.btnPickColor.Text = "Pick a color";
			this.btnPickColor.Click += new System.EventHandler(this.pickColorButton_Click);
			// 
			// tbX
			// 
			this.tbX.Location = new System.Drawing.Point(185, 120);
			this.tbX.Name = "tbX";
			this.tbX.Size = new System.Drawing.Size(75, 20);
			this.tbX.TabIndex = 5;
			this.tbX.Text = "100,0";
			// 
			// tbY
			// 
			this.tbY.Location = new System.Drawing.Point(185, 152);
			this.tbY.Name = "tbY";
			this.tbY.Size = new System.Drawing.Size(75, 20);
			this.tbY.TabIndex = 6;
			this.tbY.Text = "100,0";
			// 
			// tbZ
			// 
			this.tbZ.Location = new System.Drawing.Point(185, 184);
			this.tbZ.Name = "tbZ";
			this.tbZ.Size = new System.Drawing.Size(75, 20);
			this.tbZ.TabIndex = 7;
			this.tbZ.Text = "0,0";
			// 
			// lbX
			// 
			this.lbX.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbX.Location = new System.Drawing.Point(8, 120);
			this.lbX.Name = "lbX";
			this.lbX.Size = new System.Drawing.Size(168, 20);
			this.lbX.TabIndex = 8;
			this.lbX.Text = "X coordinate of the center :";
			// 
			// lbY
			// 
			this.lbY.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbY.Location = new System.Drawing.Point(8, 152);
			this.lbY.Name = "lbY";
			this.lbY.Size = new System.Drawing.Size(168, 20);
			this.lbY.TabIndex = 9;
			this.lbY.Text = "Y coordinate of the center :";
			// 
			// lbZ
			// 
			this.lbZ.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbZ.Location = new System.Drawing.Point(8, 184);
			this.lbZ.Name = "lbZ";
			this.lbZ.Size = new System.Drawing.Size(168, 20);
			this.lbZ.TabIndex = 10;
			this.lbZ.Text = "Z coordinate of the center :";
			// 
			// tbRadius
			// 
			this.tbRadius.Location = new System.Drawing.Point(185, 88);
			this.tbRadius.Name = "tbRadius";
			this.tbRadius.Size = new System.Drawing.Size(75, 20);
			this.tbRadius.TabIndex = 11;
			this.tbRadius.Text = "100,0";
			// 
			// lbRad
			// 
			this.lbRad.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbRad.Location = new System.Drawing.Point(8, 88);
			this.lbRad.Name = "lbRad";
			this.lbRad.Size = new System.Drawing.Size(136, 20);
			this.lbRad.TabIndex = 12;
			this.lbRad.Text = "Radius of the circle :";
			// 
			// lbWeight
			// 
			this.lbWeight.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbWeight.Location = new System.Drawing.Point(8, 56);
			this.lbWeight.Name = "lbWeight";
			this.lbWeight.Size = new System.Drawing.Size(88, 20);
			this.lbWeight.TabIndex = 14;
			this.lbWeight.Text = "Line weight :";
			// 
			// cbLineWeight
			// 
			this.cbLineWeight.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cbLineWeight.Items.AddRange(new object[] {
															  "0.1",
															  "0.2",
															  "0.3",
															  "0.4",
															  "0.5",
															  "0.6",
															  "0.7",
															  "0.8",
															  "0.9",
															  "1.0",
															  "1.1",
															  "1.2",
															  "1.3",
															  "1.4",
															  "1.5",
															  "1.6",
															  "1.7",
															  "1.8",
															  "1.9",
															  "2.0",
															  "2.1",
															  "2.2",
															  "2.3",
															  "2.4",
															  "2.5",
															  "2.6",
															  "2.7",
															  "2.8",
															  "2.9",
															  "3.0",
															  "3.1",
															  "3.2",
															  "3.3",
															  "3.4",
															  "3.5",
															  "3.6",
															  "3.7",
															  "3.8",
															  "3.9",
															  "4.0",
															  "4.1",
															  "4.2",
															  "4.3",
															  "4.4",
															  "4.5",
															  "4.6",
															  "4.7",
															  "4.8",
															  "4.9",
															  "5.0",
															  "5.1",
															  "5.2",
															  "5.3",
															  "5.4",
															  "5.5",
															  "5.6",
															  "5.7",
															  "5.8",
															  "5.9",
															  "6.0",
															  "6.1",
															  "6.2",
															  "6.3",
															  "6.4",
															  "6.5",
															  "6.6",
															  "6.7",
															  "6.8",
															  "6.9",
															  "7.0",
															  "7.1",
															  "7.2",
															  "7.3",
															  "7.4",
															  "7.5",
															  "7.6",
															  "7.7",
															  "7.8",
															  "7.9",
															  "8.0",
															  "8.1",
															  "8.2",
															  "8.3",
															  "8.4",
															  "8.5",
															  "8.6",
															  "8.7",
															  "8.8",
															  "8.9",
															  "9.0",
															  "9.1",
															  "9.2",
															  "9.3",
															  "9.4",
															  "9.5",
															  "9.6",
															  "9.7",
															  "9.8",
															  "9.9",
															  "10.0"});
			this.cbLineWeight.Location = new System.Drawing.Point(185, 56);
			this.cbLineWeight.Name = "cbLineWeight";
			this.cbLineWeight.Size = new System.Drawing.Size(75, 21);
			this.cbLineWeight.TabIndex = 15;
			// 
			// AddCircleForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(272, 251);
			this.Controls.Add(this.cbLineWeight);
			this.Controls.Add(this.lbWeight);
			this.Controls.Add(this.lbRad);
			this.Controls.Add(this.tbRadius);
			this.Controls.Add(this.tbZ);
			this.Controls.Add(this.tbY);
			this.Controls.Add(this.tbX);
			this.Controls.Add(this.tbColor);
			this.Controls.Add(this.lbZ);
			this.Controls.Add(this.lbY);
			this.Controls.Add(this.lbX);
			this.Controls.Add(this.btnPickColor);
			this.Controls.Add(this.lbColor);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MinimizeBox = false;
			this.Name = "AddCircleForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Add Circle";
			this.ResumeLayout(false);

		}
		#endregion

		private void addCircleOK_Click(object sender, System.EventArgs e)
		{
			circle = new CADCircle();
			if(setColorDlg.Color == Color.Black)
				circle.Color = CADConst.clNone;
			else
				circle.Color = setColorDlg.Color;
			try
			{
				circle.Radius = Convert.ToDouble(this.tbRadius.Text);
				circle.Point = new DPoint(Convert.ToDouble(this.tbX.Text), Convert.ToDouble(this.tbY.Text), Convert.ToDouble(this.tbZ.Text));
				circle.LineWeight = Convert.ToSingle(this.cbLineWeight.Text, ApplicationConstants.numInfo);
				cadImage.Converter.GetSection(ConvSection.Entities).AddEntity(circle);
				cadImage.Converter.OnCreate(circle);
				cadImage.Converter.Loads(circle);				
				this.OnResizeImage();
			}
			catch(FormatException exception)
			{
				MessageBox.Show(exception.Message, ApplicationConstants.appconst, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}			
			this.DialogResult = DialogResult.OK;
			this.Close();
		}

		private void addCircleCancel_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
		}

		private void pickColorButton_Click(object sender, EventArgs e)
		{
			if(setColorDlg.ShowDialog() == DialogResult.OK)			
				this.tbColor.BackColor = setColorDlg.Color;			
		}
	}
}
