using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using CADImport;
using CADImport.FaceModule;
using CADImport.CADImportForms;

namespace CADImportNetDemos.AddEntityDemo
{
	#region Help
	/// <summary>
	/// Represents a form in which the properties of the text are set 
	/// before adding it to the current CAD image.
	/// </summary>
	#endregion Help
	public class AddTextForm : System.Windows.Forms.Form
	{
		private CADText cText;
		private CADImage cadImage;		

		#region Help
		/// <summary>
		/// Invokes on changing of <see cref="CADImportNetDemos.AddEntityDemo.AddPolyForm.Image"/>image drawing area
		/// </summary>
		#endregion
		public event CADImportEventHandler ResizeImageEvent;

		private System.Windows.Forms.ColorDialog setColorDlg;		
		private System.Windows.Forms.Label line5;
		private System.Windows.Forms.Label line4;
		private System.Windows.Forms.Label line3;
		private System.Windows.Forms.Label line2;
		private System.Windows.Forms.Label line1;
		private System.Windows.Forms.Label vertLine2;
		private System.Windows.Forms.Label vertLine1;
		private System.Windows.Forms.Label RAlab;
		private System.Windows.Forms.Label lWlab;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button btnPickColor;
		private System.Windows.Forms.TextBox tbColor;
		private System.Windows.Forms.Label lbColor;
		private System.Windows.Forms.Label lbY;
		private System.Windows.Forms.Label lbZ;
		private System.Windows.Forms.Label lbX;
		private System.Windows.Forms.TextBox tbZ;
		private System.Windows.Forms.TextBox tbY;
		private System.Windows.Forms.TextBox tbX;
		private System.Windows.Forms.Label lbFirst;
		private System.Windows.Forms.Button btCancel;
		private System.Windows.Forms.Button btOK;
		private System.Windows.Forms.TextBox tbRotation;
		private System.Windows.Forms.ComboBox cbHeights;
		private System.Windows.Forms.TextBox tbText;
		private System.Windows.Forms.Label label2;		
		
		#region Help
		/// <summary>
		/// Raises the <see cref="CADImportNetDemos.AddEntityDemo.AddTextForm.ResizeImageEvent"/>ResizeImageEvent event. 
		/// </summary>
		#endregion
		public void OnResizeImage()
		{
			if (ResizeImageEvent != null)
				ResizeImageEvent();
		}

		#region Help
		/// <summary>
		/// Gets or sets <see cref="CADImport.CADImage">CADImage</see> object where new entities are added
		/// </summary>
		#endregion
		public CADImage Image
		{
			get
			{
				return this.cadImage;
			}
			set
			{
				this.cadImage = value;
			}
		}
		
		private System.ComponentModel.Container components = null;

		#region Help
		/// <summary>
		/// Initializes a new instance of the <see cref=" CADImportNetDemos.AddEntityDemo.AddTextForm">AddTextForm</see> class.
		/// </summary>
		#endregion Help
		public AddTextForm()
		{
			InitializeComponent();
			setColorDlg = new ColorDialog();
			this.cbHeights.SelectedIndex = 0;
			this.tbColor.Enabled = false;
			this.StartPosition = FormStartPosition.CenterScreen;
		}

		#region Help
		/// <summary>
		/// Cleans up any resources being used.
		/// </summary>
		/// <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
		///</param>
		#endregion Help
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnPickColor = new System.Windows.Forms.Button();
			this.tbColor = new System.Windows.Forms.TextBox();
			this.lbColor = new System.Windows.Forms.Label();
			this.RAlab = new System.Windows.Forms.Label();
			this.line5 = new System.Windows.Forms.Label();
			this.line4 = new System.Windows.Forms.Label();
			this.line3 = new System.Windows.Forms.Label();
			this.line2 = new System.Windows.Forms.Label();
			this.line1 = new System.Windows.Forms.Label();
			this.lbY = new System.Windows.Forms.Label();
			this.lbZ = new System.Windows.Forms.Label();
			this.lbX = new System.Windows.Forms.Label();
			this.tbZ = new System.Windows.Forms.TextBox();
			this.tbY = new System.Windows.Forms.TextBox();
			this.tbX = new System.Windows.Forms.TextBox();
			this.vertLine2 = new System.Windows.Forms.Label();
			this.vertLine1 = new System.Windows.Forms.Label();
			this.lbFirst = new System.Windows.Forms.Label();
			this.btCancel = new System.Windows.Forms.Button();
			this.btOK = new System.Windows.Forms.Button();
			this.tbRotation = new System.Windows.Forms.TextBox();
			this.cbHeights = new System.Windows.Forms.ComboBox();
			this.lWlab = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.tbText = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// btnPickColor
			// 
			this.btnPickColor.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnPickColor.Location = new System.Drawing.Point(136, 8);
			this.btnPickColor.Name = "btnPickColor";
			this.btnPickColor.Size = new System.Drawing.Size(72, 25);
			this.btnPickColor.TabIndex = 10;
			this.btnPickColor.Text = "Pick a color";
			this.btnPickColor.Click += new System.EventHandler(this.btnPickColor_Click);
			// 
			// tbColor
			// 
			this.tbColor.BackColor = System.Drawing.Color.Black;
			this.tbColor.Location = new System.Drawing.Point(56, 8);
			this.tbColor.Multiline = true;
			this.tbColor.Name = "tbColor";
			this.tbColor.Size = new System.Drawing.Size(56, 23);
			this.tbColor.TabIndex = 9;
			this.tbColor.Text = "";
			// 
			// lbColor
			// 
			this.lbColor.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbColor.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.lbColor.Location = new System.Drawing.Point(8, 8);
			this.lbColor.Name = "lbColor";
			this.lbColor.Size = new System.Drawing.Size(48, 23);
			this.lbColor.TabIndex = 8;
			this.lbColor.Text = "Color :";
			// 
			// RAlab
			// 
			this.RAlab.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.RAlab.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.RAlab.Location = new System.Drawing.Point(8, 168);
			this.RAlab.Name = "RAlab";
			this.RAlab.Size = new System.Drawing.Size(104, 20);
			this.RAlab.TabIndex = 31;
			this.RAlab.Text = "Rotation angle :";
			// 
			// line5
			// 
			this.line5.BackColor = System.Drawing.Color.Black;
			this.line5.Location = new System.Drawing.Point(10, 160);
			this.line5.Name = "line5";
			this.line5.Size = new System.Drawing.Size(198, 1);
			this.line5.TabIndex = 37;
			// 
			// line4
			// 
			this.line4.BackColor = System.Drawing.Color.Black;
			this.line4.Location = new System.Drawing.Point(10, 128);
			this.line4.Name = "line4";
			this.line4.Size = new System.Drawing.Size(198, 1);
			this.line4.TabIndex = 36;
			// 
			// line3
			// 
			this.line3.BackColor = System.Drawing.Color.Black;
			this.line3.Location = new System.Drawing.Point(10, 96);
			this.line3.Name = "line3";
			this.line3.Size = new System.Drawing.Size(198, 1);
			this.line3.TabIndex = 35;
			// 
			// line2
			// 
			this.line2.BackColor = System.Drawing.Color.Black;
			this.line2.Location = new System.Drawing.Point(10, 64);
			this.line2.Name = "line2";
			this.line2.Size = new System.Drawing.Size(198, 1);
			this.line2.TabIndex = 34;
			// 
			// line1
			// 
			this.line1.BackColor = System.Drawing.Color.Black;
			this.line1.Location = new System.Drawing.Point(10, 40);
			this.line1.Name = "line1";
			this.line1.Size = new System.Drawing.Size(198, 1);
			this.line1.TabIndex = 33;
			// 
			// lbY
			// 
			this.lbY.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbY.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbY.Location = new System.Drawing.Point(10, 104);
			this.lbY.Name = "lbY";
			this.lbY.Size = new System.Drawing.Size(98, 20);
			this.lbY.TabIndex = 39;
			this.lbY.Text = "Y coordinate :";
			this.lbY.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lbZ
			// 
			this.lbZ.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbZ.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbZ.Location = new System.Drawing.Point(10, 136);
			this.lbZ.Name = "lbZ";
			this.lbZ.Size = new System.Drawing.Size(98, 20);
			this.lbZ.TabIndex = 40;
			this.lbZ.Text = "Z coordinate :";
			this.lbZ.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// lbX
			// 
			this.lbX.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbX.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lbX.Location = new System.Drawing.Point(10, 72);
			this.lbX.Name = "lbX";
			this.lbX.Size = new System.Drawing.Size(98, 20);
			this.lbX.TabIndex = 38;
			this.lbX.Text = "X coordinate :";
			this.lbX.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// tbZ
			// 
			this.tbZ.Location = new System.Drawing.Point(122, 136);
			this.tbZ.Name = "tbZ";
			this.tbZ.Size = new System.Drawing.Size(76, 20);
			this.tbZ.TabIndex = 43;
			this.tbZ.Text = "0,0";
			// 
			// tbY
			// 
			this.tbY.Location = new System.Drawing.Point(122, 104);
			this.tbY.Name = "tbY";
			this.tbY.Size = new System.Drawing.Size(76, 20);
			this.tbY.TabIndex = 42;
			this.tbY.Text = "0,0";
			// 
			// tbX
			// 
			this.tbX.Location = new System.Drawing.Point(122, 72);
			this.tbX.Name = "tbX";
			this.tbX.Size = new System.Drawing.Size(76, 20);
			this.tbX.TabIndex = 41;
			this.tbX.Text = "0,0";
			// 
			// vertLine2
			// 
			this.vertLine2.BackColor = System.Drawing.Color.Black;
			this.vertLine2.Location = new System.Drawing.Point(207, 40);
			this.vertLine2.Name = "vertLine2";
			this.vertLine2.Size = new System.Drawing.Size(1, 120);
			this.vertLine2.TabIndex = 45;
			// 
			// vertLine1
			// 
			this.vertLine1.BackColor = System.Drawing.Color.Black;
			this.vertLine1.Location = new System.Drawing.Point(106, 40);
			this.vertLine1.Name = "vertLine1";
			this.vertLine1.Size = new System.Drawing.Size(1, 120);
			this.vertLine1.TabIndex = 44;
			// 
			// lbFirst
			// 
			this.lbFirst.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lbFirst.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.lbFirst.Location = new System.Drawing.Point(122, 45);
			this.lbFirst.Name = "lbFirst";
			this.lbFirst.Size = new System.Drawing.Size(68, 16);
			this.lbFirst.TabIndex = 46;
			this.lbFirst.Text = "Start point";
			// 
			// btCancel
			// 
			this.btCancel.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btCancel.Location = new System.Drawing.Point(133, 320);
			this.btCancel.Name = "btCancel";
			this.btCancel.Size = new System.Drawing.Size(75, 25);
			this.btCancel.TabIndex = 48;
			this.btCancel.Text = "Cancel";
			this.btCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// btOK
			// 
			this.btOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btOK.Location = new System.Drawing.Point(48, 320);
			this.btOK.Name = "btOK";
			this.btOK.Size = new System.Drawing.Size(75, 25);
			this.btOK.TabIndex = 47;
			this.btOK.Text = "OK";
			this.btOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// tbRotation
			// 
			this.tbRotation.Location = new System.Drawing.Point(120, 168);
			this.tbRotation.Name = "tbRotation";
			this.tbRotation.Size = new System.Drawing.Size(76, 20);
			this.tbRotation.TabIndex = 49;
			this.tbRotation.Text = "0";
			// 
			// cbHeights
			// 
			this.cbHeights.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.cbHeights.Items.AddRange(new object[] {
														   "1",
														   "2",
														   "3",
														   "4",
														   "5",
														   "6",
														   "7",
														   "8",
														   "9",
														   "10",
														   "11",
														   "12",
														   "13",
														   "14",
														   "15",
														   "16",
														   "17",
														   "18",
														   "19",
														   "20",
														   "21",
														   "22",
														   "23",
														   "24",
														   "25",
														   "26",
														   "27",
														   "28",
														   "29",
														   "30",
														   "31",
														   "32",
														   "33",
														   "34",
														   "35",
														   "36",
														   "37",
														   "38",
														   "39",
														   "40",
														   "41",
														   "42",
														   "43",
														   "44",
														   "45",
														   "46",
														   "47",
														   "48",
														   "49",
														   "50",
														   "51",
														   "52",
														   "53",
														   "54",
														   "55",
														   "56",
														   "57",
														   "58",
														   "59",
														   "60",
														   "61",
														   "62",
														   "63",
														   "64",
														   "65",
														   "66",
														   "67",
														   "68",
														   "69",
														   "70",
														   "71",
														   "72",
														   "73",
														   "74",
														   "75",
														   "76",
														   "77",
														   "78",
														   "79",
														   "80",
														   "81",
														   "82",
														   "83",
														   "84",
														   "85",
														   "86",
														   "87",
														   "88",
														   "89",
														   "90",
														   "91",
														   "92",
														   "93",
														   "94",
														   "95",
														   "96",
														   "97",
														   "98",
														   "99",
														   "100"});
			this.cbHeights.Location = new System.Drawing.Point(120, 192);
			this.cbHeights.Name = "cbHeights";
			this.cbHeights.Size = new System.Drawing.Size(76, 21);
			this.cbHeights.TabIndex = 51;
			// 
			// lWlab
			// 
			this.lWlab.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.lWlab.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lWlab.Location = new System.Drawing.Point(8, 192);
			this.lWlab.Name = "lWlab";
			this.lWlab.Size = new System.Drawing.Size(104, 20);
			this.lWlab.TabIndex = 50;
			this.lWlab.Text = "Text height :";
			// 
			// label1
			// 
			this.label1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.label1.Location = new System.Drawing.Point(8, 216);
			this.label1.Name = "label1";
			this.label1.TabIndex = 52;
			this.label1.Text = "Text contents :";
			// 
			// tbText
			// 
			this.tbText.Location = new System.Drawing.Point(8, 240);
			this.tbText.Multiline = true;
			this.tbText.Name = "tbText";
			this.tbText.Size = new System.Drawing.Size(200, 72);
			this.tbText.TabIndex = 53;
			this.tbText.Text = "";
			// 
			// label2
			// 
			this.label2.BackColor = System.Drawing.Color.Black;
			this.label2.Location = new System.Drawing.Point(10, 40);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(1, 120);
			this.label2.TabIndex = 54;
			// 
			// AddTextForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(216, 352);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.tbText);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.cbHeights);
			this.Controls.Add(this.lWlab);
			this.Controls.Add(this.tbRotation);
			this.Controls.Add(this.btCancel);
			this.Controls.Add(this.btOK);
			this.Controls.Add(this.lbFirst);
			this.Controls.Add(this.vertLine2);
			this.Controls.Add(this.vertLine1);
			this.Controls.Add(this.tbZ);
			this.Controls.Add(this.tbY);
			this.Controls.Add(this.tbX);
			this.Controls.Add(this.lbY);
			this.Controls.Add(this.lbZ);
			this.Controls.Add(this.lbX);
			this.Controls.Add(this.line5);
			this.Controls.Add(this.line4);
			this.Controls.Add(this.line3);
			this.Controls.Add(this.line2);
			this.Controls.Add(this.line1);
			this.Controls.Add(this.RAlab);
			this.Controls.Add(this.btnPickColor);
			this.Controls.Add(this.tbColor);
			this.Controls.Add(this.lbColor);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.Name = "AddTextForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Add Text";
			this.ResumeLayout(false);

		}
		#endregion

		private void btnPickColor_Click(object sender, System.EventArgs e)
		{
			if(setColorDlg.ShowDialog() == DialogResult.OK)			
				this.tbColor.BackColor = setColorDlg.Color;			
		}

		private void btnOK_Click(object sender, System.EventArgs e)
		{
			cText = new CADText();
			if(setColorDlg.Color == Color.Black)
				cText.Color = CADConst.clNone;
			else
				cText.Color = setColorDlg.Color;
			try
			{
				cText.Point = new DPoint(Convert.ToDouble(this.tbX.Text), Convert.ToDouble(this.tbY.Text), Convert.ToDouble(this.tbZ.Text));
				cText.Height = Convert.ToInt16(this.cbHeights.Text);
				cText.Rotation = Convert.ToInt32(this.tbRotation.Text);
				cText.Text = this.tbText.Text;
				cadImage.Converter.GetSection(ConvSection.Entities).AddEntity(cText);
				cadImage.Converter.OnCreate(cText);
				cadImage.Converter.Loads(cText);				
				this.OnResizeImage();
			}
			catch(FormatException exception)
			{
				MessageBox.Show(exception.Message, ApplicationConstants.appnamestr, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
			}						
			this.DialogResult = DialogResult.OK;
			this.Close();
		}

		private void btnCancel_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
		}		
	}
}
