Imports System
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports CADImport
Imports CADImport.FaceModule
Imports CADImport.CADImportForms
''' <summary>
''' Represents a form in which the properties of the polyline are set 
''' before adding it to the current CAD image.
''' </summary>
Public Class AddPolyForm
    Inherits Form
    ' Events
    Public Event ResizeImageEvent As CADImportEventHandler

    ' Methods
    ''' <summary>
    ''' Initializes a new instance of the <see cref="!:AddEntityDemoApplication.AddPolyForm">AddPolyForm</see> class.
    ''' </summary>
    Public Sub New()
        Me.InitializeComponent()
        Me.setColorDlg = New ColorDialog
        Me.SetDefaut()
    End Sub

    Private Sub addPolyCancel_Click(ByVal sender As Object, ByVal e As EventArgs)
        MyBase.DialogResult = DialogResult.Cancel
        MyBase.Close()
    End Sub

    Private Sub AddPolyForm_Load(ByVal sender As Object, ByVal e As EventArgs)
        If (Me.poly.Entities.Count <> 0) Then
            Me.SetDefaut()
        End If
    End Sub

    Private Sub addPolyOK_Click(ByVal sender As Object, ByVal e As EventArgs)
        If (Me.poly.Entities.Count = 0) Then
            MessageBox.Show(ApplicationConstants.msgstr2, ApplicationConstants.appnamestr, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        Else
            If (Me.setColorDlg.Color.Equals(Color.Black)) Then
                Me.poly.Color = CADConst.clNone
            Else
                Me.poly.Color = Me.setColorDlg.Color
            End If
            Try
                Me.poly.LineWeight = Convert.ToSingle(Me.cbLineWeight.Text, ApplicationConstants.numInfo)
                Me.cadImage.Converter.GetSection(ConvSection.Entities).AddEntity(Me.poly)
                Me.cadImage.Converter.OnCreate(Me.poly)
                Me.cadImage.Converter.Loads(Me.poly)
                Me.OnResizeImage()
            Catch exception As FormatException
                MessageBox.Show(exception.Message, ApplicationConstants.appnamestr, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End Try
            MyBase.DialogResult = DialogResult.OK
            MyBase.Close()
        End If
    End Sub

    Private Sub addVertBut_Click(ByVal sender As Object, ByVal e As EventArgs)
        If (Me.poly.Entities.Count = 0) Then
            Me.vert0 = New CADVertex
            Me.vertNext = New CADVertex
            Try
                Me.vert0.Point = New DPoint(Convert.ToDouble(Me.tbX.Text), Convert.ToDouble(Me.tbY.Text), Convert.ToDouble(Me.tbZ.Text))
                Me.poly.Entities.Add(Me.vert0)
                Me.tbVertNum.Text = Me.poly.Entities.Count.ToString
                Me.tbX.Enabled = False
                Me.tbY.Enabled = False
                Me.tbZ.Enabled = False
                Me.tbX2.Enabled = True
                Me.tbY2.Enabled = True
                Me.tbZ2.Enabled = True
            Catch exception As FormatException
                MessageBox.Show(exception.Message, ApplicationConstants.appnamestr, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End Try
        Else
            Me.vertNext = New CADVertex
            Try
                Me.vertNext.Point = New DPoint(Convert.ToDouble(Me.tbX2.Text), Convert.ToDouble(Me.tbY2.Text), Convert.ToDouble(Me.tbZ2.Text))
                Me.poly.Entities.Add(Me.vertNext)
                Me.tbVertNum.Text = Me.poly.Entities.Count.ToString
            Catch exception As FormatException
                MessageBox.Show(exception.Message, ApplicationConstants.appnamestr, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End Try
        End If
    End Sub

    ''' <summary>
    ''' Cleans up any resources being used.
    ''' </summary>
    ''' <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
    ''' </param>
    Protected Overloads Sub Dispose(ByVal disposing As Boolean)
        If (disposing AndAlso (Not Me.components Is Nothing)) Then
            Me.components.Dispose()
        End If
        MyBase.Dispose(disposing)
    End Sub

#Region " Windows Form Designer generated code "
    ''' <summary>
    ''' Required method for Designer support - do not modify
    ''' the contents of this method with the code editor.
    ''' </summary>
    Private Sub InitializeComponent()
        Me.btnPickColor = New Button
        Me.tbColor = New TextBox
        Me.lbColor = New Label
        Me.lbY = New Label
        Me.lbZ = New Label
        Me.lbX = New Label
        Me.tbZ2 = New TextBox
        Me.tbY2 = New TextBox
        Me.tbX2 = New TextBox
        Me.tbZ = New TextBox
        Me.tbY = New TextBox
        Me.tbX = New TextBox
        Me.lbSec = New Label
        Me.lbFirst = New Label
        Me.line5 = New Label
        Me.line4 = New Label
        Me.line3 = New Label
        Me.line2 = New Label
        Me.line1 = New Label
        Me.vertLine2 = New Label
        Me.vertLine1 = New Label
        Me.lbNum = New Label
        Me.tbVertNum = New TextBox
        Me.btnAddVert = New Button
        Me.cbLineWeight = New ComboBox
        Me.lbLineWeight = New Label
        Me.btnCancel = New Button
        Me.btnOK = New Button
        Me.label1 = New Label
        Me.label2 = New Label
        MyBase.SuspendLayout()
        Me.btnPickColor.FlatStyle = FlatStyle.System
        Me.btnPickColor.Location = New Point(&HCC, 8)
        Me.btnPickColor.Name = "btnPickColor"
        Me.btnPickColor.Size = New Size(&H4B, &H19)
        Me.btnPickColor.TabIndex = 10
        Me.btnPickColor.Text = "Pick a color"
        AddHandler Me.btnPickColor.Click, New System.EventHandler(AddressOf Me.pickColorButton_Click)
        Me.tbColor.BackColor = Color.Black
        Me.tbColor.Location = New Point(&H68, 8)
        Me.tbColor.Multiline = True
        Me.tbColor.Name = "tbColor"
        Me.tbColor.Size = New Size(&H48, &H17)
        Me.tbColor.TabIndex = 9
        Me.tbColor.Text = ""
        Me.lbColor.FlatStyle = FlatStyle.System
        Me.lbColor.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
        Me.lbColor.Location = New Point(8, 8)
        Me.lbColor.Name = "lbColor"
        Me.lbColor.Size = New Size(&H38, &H17)
        Me.lbColor.TabIndex = 8
        Me.lbColor.Text = "Color :"
        Me.lbY.FlatStyle = FlatStyle.System
        Me.lbY.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, 0)
        Me.lbY.Location = New Point(8, &H88)
        Me.lbY.Name = "lbY"
        Me.lbY.Size = New Size(&H62, 20)
        Me.lbY.TabIndex = &H12
        Me.lbY.Text = "Y coordinate :"
        Me.lbY.TextAlign = ContentAlignment.MiddleCenter
        Me.lbZ.FlatStyle = FlatStyle.System
        Me.lbZ.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, 0)
        Me.lbZ.Location = New Point(8, &HA8)
        Me.lbZ.Name = "lbZ"
        Me.lbZ.Size = New Size(&H62, 20)
        Me.lbZ.TabIndex = &H13
        Me.lbZ.Text = "Z coordinate :"
        Me.lbZ.TextAlign = ContentAlignment.MiddleCenter
        Me.lbX.FlatStyle = FlatStyle.System
        Me.lbX.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, 0)
        Me.lbX.Location = New Point(8, &H68)
        Me.lbX.Name = "lbX"
        Me.lbX.Size = New Size(&H62, 20)
        Me.lbX.TabIndex = &H11
        Me.lbX.Text = "X coordinate :"
        Me.lbX.TextAlign = ContentAlignment.MiddleCenter
        Me.tbZ2.Location = New Point(&HD0, &HA8)
        Me.tbZ2.Name = "tbZ2"
        Me.tbZ2.Size = New Size(&H38, 20)
        Me.tbZ2.TabIndex = &H19
        Me.tbZ2.Text = "0,0"
        Me.tbY2.Location = New Point(&HD0, &H88)
        Me.tbY2.Name = "tbY2"
        Me.tbY2.Size = New Size(&H38, 20)
        Me.tbY2.TabIndex = &H18
        Me.tbY2.Text = "0,0"
        Me.tbX2.Location = New Point(&HD0, &H68)
        Me.tbX2.Name = "tbX2"
        Me.tbX2.Size = New Size(&H38, 20)
        Me.tbX2.TabIndex = &H17
        Me.tbX2.Text = "0,0"
        Me.tbZ.Location = New Point(120, &HA8)
        Me.tbZ.Name = "tbZ"
        Me.tbZ.Size = New Size(&H38, 20)
        Me.tbZ.TabIndex = &H16
        Me.tbZ.Text = "0,0"
        Me.tbY.Location = New Point(120, &H88)
        Me.tbY.Name = "tbY"
        Me.tbY.Size = New Size(&H38, 20)
        Me.tbY.TabIndex = &H15
        Me.tbY.Text = "0,0"
        Me.tbX.Location = New Point(120, &H68)
        Me.tbX.Name = "tbX"
        Me.tbX.Size = New Size(&H38, 20)
        Me.tbX.TabIndex = 20
        Me.tbX.Text = "0,0"
        Me.lbSec.FlatStyle = FlatStyle.System
        Me.lbSec.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
        Me.lbSec.Location = New Point(&HC7, &H4C)
        Me.lbSec.Name = "lbSec"
        Me.lbSec.Size = New Size(&H58, &H17)
        Me.lbSec.TabIndex = &H1B
        Me.lbSec.Text = "Next vertex"
        Me.lbFirst.FlatStyle = FlatStyle.System
        Me.lbFirst.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
        Me.lbFirst.Location = New Point(&H6D, &H4C)
        Me.lbFirst.Name = "lbFirst"
        Me.lbFirst.Size = New Size(&H58, &H17)
        Me.lbFirst.TabIndex = &H1A
        Me.lbFirst.Text = "First vertex"
        Me.line5.BackColor = Color.Black
        Me.line5.Location = New Point(12, &HC0)
        Me.line5.Name = "line5"
        Me.line5.Size = New Size(&H109, 1)
        Me.line5.TabIndex = &H20
        Me.line4.BackColor = Color.Black
        Me.line4.Location = New Point(12, 160)
        Me.line4.Name = "line4"
        Me.line4.Size = New Size(&H10A, 1)
        Me.line4.TabIndex = &H1F
        Me.line3.BackColor = Color.Black
        Me.line3.Location = New Point(12, &H80)
        Me.line3.Name = "line3"
        Me.line3.Size = New Size(&H10A, 1)
        Me.line3.TabIndex = 30
        Me.line2.BackColor = Color.Black
        Me.line2.Location = New Point(12, &H60)
        Me.line2.Name = "line2"
        Me.line2.Size = New Size(&H10A, 1)
        Me.line2.TabIndex = &H1D
        Me.line1.BackColor = Color.Black
        Me.line1.Location = New Point(12, &H48)
        Me.line1.Name = "line1"
        Me.line1.Size = New Size(&H109, 1)
        Me.line1.TabIndex = &H1C
        Me.vertLine2.BackColor = Color.Black
        Me.vertLine2.Location = New Point(&HB8, &H48)
        Me.vertLine2.Name = "vertLine2"
        Me.vertLine2.Size = New Size(1, 120)
        Me.vertLine2.TabIndex = &H22
        Me.vertLine1.BackColor = Color.Black
        Me.vertLine1.Location = New Point(&H68, &H48)
        Me.vertLine1.Name = "vertLine1"
        Me.vertLine1.Size = New Size(1, 120)
        Me.vertLine1.TabIndex = &H21
        Me.lbNum.FlatStyle = FlatStyle.System
        Me.lbNum.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
        Me.lbNum.Location = New Point(8, &HD0)
        Me.lbNum.Name = "lbNum"
        Me.lbNum.Size = New Size(&H80, &H17)
        Me.lbNum.TabIndex = &H23
        Me.lbNum.Text = "Number of vertices :"
        Me.tbVertNum.Location = New Point(&H88, &HD0)
        Me.tbVertNum.Name = "tbVertNum"
        Me.tbVertNum.Size = New Size(&H38, 20)
        Me.tbVertNum.TabIndex = &H24
        Me.tbVertNum.Text = ""
        Me.btnAddVert.FlatStyle = FlatStyle.System
        Me.btnAddVert.Location = New Point(&HCC, 40)
        Me.btnAddVert.Name = "btnAddVert"
        Me.btnAddVert.Size = New Size(&H4B, &H19)
        Me.btnAddVert.TabIndex = &H25
        Me.btnAddVert.Text = "Add vertex"
        AddHandler Me.btnAddVert.Click, New System.EventHandler(AddressOf Me.addVertBut_Click)
        Me.cbLineWeight.DropDownStyle = ComboBoxStyle.DropDownList
        Me.cbLineWeight.Items.AddRange(New Object() {"0.1", "0.2", "0.3", "0.4", "0.5", "0.6", "0.7", "0.8", "0.9", "1.0", "1.1", "1.2", "1.3", "1.4", "1.5", "1.6", "1.7", "1.8", "1.9", "2.0", "2.1", "2.2", "2.3", "2.4", "2.5", "2.6", "2.7", "2.8", "2.9", "3.0", "3.1", "3.2", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9", "4.0", "4.1", "4.2", "4.3", "4.4", "4.5", "4.6", "4.7", "4.8", "4.9", "5.0", "5.1", "5.2", "5.3", "5.4", "5.5", "5.6", "5.7", "5.8", "5.9", "6.0", "6.1", "6.2", "6.3", "6.4", "6.5", "6.6", "6.7", "6.8", "6.9", "7.0", "7.1", "7.2", "7.3", "7.4", "7.5", "7.6", "7.7", "7.8", "7.9", "8.0", "8.1", "8.2", "8.3", "8.4", "8.5", "8.6", "8.7", "8.8", "8.9", "9.0", "9.1", "9.2", "9.3", "9.4", "9.5", "9.6", "9.7", "9.8", "9.9", "10.0"})
        Me.cbLineWeight.Location = New Point(&H68, 40)
        Me.cbLineWeight.Name = "cbLineWeight"
        Me.cbLineWeight.Size = New Size(&H48, &H15)
        Me.cbLineWeight.TabIndex = &H27
        Me.lbLineWeight.FlatStyle = FlatStyle.System
        Me.lbLineWeight.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, 0)
        Me.lbLineWeight.Location = New Point(8, 40)
        Me.lbLineWeight.Name = "lbLineWeight"
        Me.lbLineWeight.Size = New Size(&H58, 20)
        Me.lbLineWeight.TabIndex = &H26
        Me.lbLineWeight.Text = "Line weight :"
        Me.btnCancel.FlatStyle = FlatStyle.System
        Me.btnCancel.Location = New Point(&HCD, &HEE)
        Me.btnCancel.Name = "btnCancel"
        Me.btnCancel.Size = New Size(&H4B, &H19)
        Me.btnCancel.TabIndex = &H29
        Me.btnCancel.Text = "Cancel"
        AddHandler Me.btnCancel.Click, New System.EventHandler(AddressOf Me.addPolyCancel_Click)
        Me.btnOK.FlatStyle = FlatStyle.System
        Me.btnOK.Location = New Point(&H7D, &HEE)
        Me.btnOK.Name = "btnOK"
        Me.btnOK.Size = New Size(&H4B, &H19)
        Me.btnOK.TabIndex = 40
        Me.btnOK.Text = "OK"
        AddHandler Me.btnOK.Click, New System.EventHandler(AddressOf Me.addPolyOK_Click)
        Me.label1.BackColor = Color.Black
        Me.label1.Location = New Point(11, &H49)
        Me.label1.Name = "label1"
        Me.label1.Size = New Size(1, 120)
        Me.label1.TabIndex = &H2A
        Me.label2.BackColor = Color.Black
        Me.label2.Location = New Point(&H115, &H49)
        Me.label2.Name = "label2"
        Me.label2.Size = New Size(1, 120)
        Me.label2.TabIndex = &H2B
        Me.AutoScaleBaseSize = New Size(5, 13)
        MyBase.ClientSize = New Size(&H120, &H10B)
        MyBase.Controls.Add(Me.label2)
        MyBase.Controls.Add(Me.label1)
        MyBase.Controls.Add(Me.btnCancel)
        MyBase.Controls.Add(Me.btnOK)
        MyBase.Controls.Add(Me.cbLineWeight)
        MyBase.Controls.Add(Me.lbLineWeight)
        MyBase.Controls.Add(Me.btnAddVert)
        MyBase.Controls.Add(Me.tbVertNum)
        MyBase.Controls.Add(Me.tbZ2)
        MyBase.Controls.Add(Me.tbY2)
        MyBase.Controls.Add(Me.tbX2)
        MyBase.Controls.Add(Me.tbZ)
        MyBase.Controls.Add(Me.tbY)
        MyBase.Controls.Add(Me.tbX)
        MyBase.Controls.Add(Me.tbColor)
        MyBase.Controls.Add(Me.lbNum)
        MyBase.Controls.Add(Me.vertLine2)
        MyBase.Controls.Add(Me.vertLine1)
        MyBase.Controls.Add(Me.line5)
        MyBase.Controls.Add(Me.line4)
        MyBase.Controls.Add(Me.line3)
        MyBase.Controls.Add(Me.line2)
        MyBase.Controls.Add(Me.line1)
        MyBase.Controls.Add(Me.lbFirst)
        MyBase.Controls.Add(Me.lbY)
        MyBase.Controls.Add(Me.lbZ)
        MyBase.Controls.Add(Me.lbX)
        MyBase.Controls.Add(Me.btnPickColor)
        MyBase.Controls.Add(Me.lbColor)
        MyBase.Controls.Add(Me.lbSec)
        MyBase.FormBorderStyle = FormBorderStyle.FixedSingle
        MyBase.MaximizeBox = False
        MyBase.Name = "AddPolyForm"
        MyBase.StartPosition = FormStartPosition.CenterParent
        Me.Text = "Add Polyline"
        AddHandler MyBase.Load, New System.EventHandler(AddressOf Me.AddPolyForm_Load)
        MyBase.ResumeLayout(False)
    End Sub
#End Region

    Public Sub OnResizeImage()
        'If (Not Me.ResizeImageEvent Is Nothing) Then
            RaiseEvent ResizeImageEvent()
        'End If
    End Sub

    Private Sub pickColorButton_Click(ByVal sender As Object, ByVal e As EventArgs)
        If (Me.setColorDlg.ShowDialog = DialogResult.OK) Then
            Me.tbColor.BackColor = Me.setColorDlg.Color
        End If
    End Sub

    Private Sub SetDefaut()
        Me.cbLineWeight.SelectedItem = ApplicationConstants.onestr
        Me.tbColor.Enabled = False
        Me.tbVertNum.ReadOnly = True
        Me.poly = New CADPolyLine
        Me.tbVertNum.BackColor = Color.White
        Me.tbVertNum.Text = Me.poly.Entities.Count.ToString
        MyBase.StartPosition = FormStartPosition.CenterScreen
        Me.tbX2.Enabled = False
        Me.tbY2.Enabled = False
        Me.tbZ2.Enabled = False
        Me.tbX.Enabled = True
        Me.tbY.Enabled = True
        Me.tbZ.Enabled = True
    End Sub


    ' Properties
    Public Property Image() As CADImage
        Get
            Return Me.cadImage
        End Get
        Set(ByVal value As CADImage)
            Me.cadImage = value
        End Set
    End Property


    ' Fields
    Private btnAddVert As Button
    Private btnCancel As Button
    Private btnOK As Button
    Private btnPickColor As Button
    Private cadImage As CADImage
    Private cbLineWeight As ComboBox
    Private components As Container = Nothing
    Private label1 As Label
    Private label2 As Label
    Private lbColor As Label
    Private lbFirst As Label
    Private lbLineWeight As Label
    Private lbNum As Label
    Private lbSec As Label
    Private lbX As Label
    Private lbY As Label
    Private lbZ As Label
    Private line1 As Label
    Private line2 As Label
    Private line3 As Label
    Private line4 As Label
    Private line5 As Label
    Private poly As CADPolyLine
    Private setColorDlg As ColorDialog
    Private tbColor As TextBox
    Private tbVertNum As TextBox
    Private tbX As TextBox
    Private tbX2 As TextBox
    Private tbY As TextBox
    Private tbY2 As TextBox
    Private tbZ As TextBox
    Private tbZ2 As TextBox
    Private vert0 As CADVertex
    Private vertLine1 As Label
    Private vertLine2 As Label
    Private vertNext As CADVertex
End Class
