//To compile the project you need the extended version of the CADImport .NET library
//that allows to access CAD entities directly.  You can get this version of the 
//library free of charge by emailing your request to: info@cadsofttools.com
using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Globalization;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using CADImport;
using CADImport.CADImportForms;
using CADImport.FaceModule;
using CADImport.RasterImage;
using CADImport.Professional;
using CADImport.HPGL2;

namespace CADImportNetDemos.CADEditorDemo
{
	#region Help
	#if RUSHELP
	/// <summary>
	///        CAD 
	/// </summary>
	#else
	/// <summary>
	/// Represents the main application form where CAD image and added CAD entities can be viewed.
	/// </summary>
	#endif
	#endregion Help
	public class MainForm: System.Windows.Forms.Form
	{			
		private CADImport.CADImportForms.LayerForm lForm;		
		private CADImport.CADImportForms.AboutForm aboutForm;
		private CADImport.CADImportForms.SHXForm shxFrm;
		private CADImport.Printing.PrintingForm prtForm;		
		private CADImport.CADImportForms.SetRasterSizeForm rasterSizeForm;
		#region protect
#if protect
		private RegForm regFrm;
#endif
		#if (protect && floatprotect)
			private CADImport.CADImportForms.FloatLicForm floatLicFrm;
		#endif
		#endregion protect		

		private PointF old_Pos;	
		private PointF pos;
		private SizeF visibleArea;
		private float sc;				
		private int cX, cY;
		private bool detMouseDown;
		private float prev_sc;		
		private bool moveMarker;				
		private CADImage cadImage;
		private ClipRect clipRectangle;
		private MultipleLanguage mlng;
		private int curLngInd;		
		private string lngFile;
		private SortedList settingsLst;
		private SaveSettings svSet;		
		private int addEntity;
		private bool colorDraw;
		private bool dimVisible;
		private bool showLineWeight;
		private bool textVisible;
		private bool cntrlDown;			
		private bool stopSnap;
		private bool enableSnap;
		private EntitiesCreator entCreator;		
		private Point startPoint;
		private Point endPoint;		
		private CreatorType curAddEntityType;		
		private static string fileName;	
		private static readonly string fileSettingsName;
		private bool useSHXFonts;

		private CADImport.FaceModule.ListItemSelectEventHandler listItemSelectDel;
		private CADImport.FaceModule.CADPictureBox cadPictBox;

		private System.Windows.Forms.StatusBar stBar;
		private System.Windows.Forms.StatusBarPanel sbOpen;
		private System.Windows.Forms.StatusBarPanel sbScale;
		private System.Windows.Forms.ToolBarButton tlbOpen;
		private System.Windows.Forms.ToolBarButton tlbZoomIn;
		private System.Windows.Forms.ToolBarButton tlbZoomOut;
		private System.Windows.Forms.ToolBarButton tlbLay;
		private System.Windows.Forms.ToolBarButton tlbWhite;
		private System.Windows.Forms.ToolBarButton tlbBlack;
		private System.Windows.Forms.ToolBarButton tlbSave;
		private System.Windows.Forms.ToolBar tlbTool;
		private System.Windows.Forms.StatusBarPanel sbCoord;		
		private System.Windows.Forms.ImageList toolBtnImageList;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.SaveFileDialog saveImgDlg;
		private System.Windows.Forms.SaveFileDialog saveDXFDlg;
		private System.Windows.Forms.ToolBarButton tlbShx;		
		private System.Windows.Forms.MenuItem miFloatLic;								
		private System.Windows.Forms.ToolBarButton tlbAddImgEnt;
		private System.Windows.Forms.ToolBarButton tlbOptions;
		private System.Windows.Forms.ToolBarButton tlbAddSolid;
		private System.Windows.Forms.ToolBarButton tlbNew;
		private System.Windows.Forms.ToolBarButton tlbLine;
		private System.Windows.Forms.ToolBarButton tlbRect;
		private System.Windows.Forms.ToolBarButton tlbCircle;
		private System.Windows.Forms.ToolBarButton tlbText;
		private System.Windows.Forms.ToolBarButton tlbPoly;
		private System.Windows.Forms.ToolBarButton tlbPen;
		private System.Windows.Forms.ToolBarButton tlbAddMText;
		private System.Windows.Forms.TreeView trvEntity;
		private System.Windows.Forms.Splitter spProp;
		private System.Windows.Forms.Splitter spltPictBox;
		private System.Windows.Forms.ToolBarButton tlbGetExt;
		private System.Windows.Forms.PrintPreviewDialog printPrevDlg;
		private System.Windows.Forms.MenuItem miLng;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.Panel trvPanel;		
		private System.Windows.Forms.ToolBarButton tlbPoint;
		private System.Windows.Forms.ToolBarButton tlbDel;
		private System.Windows.Forms.ToolBarButton tlbUndo;
		private System.Windows.Forms.ToolBarButton tlbRedo;			
		private System.Windows.Forms.PropertyGrid propGrid;				
		private System.Windows.Forms.MenuItem miUndo;
		private System.Windows.Forms.MenuItem miRedo;
		private System.Windows.Forms.MenuItem miDelete;
		private System.Windows.Forms.MenuItem miPrintCustom;		
		private System.Windows.Forms.ColorDialog setColorDlg;
		private System.Windows.Forms.ToolBarButton tlbCloud;
		private System.Windows.Forms.ToolBarButton tlbAddHatch;
		private System.Windows.Forms.MenuItem miCopy;
		private System.Windows.Forms.MenuItem miCut;
		private System.Windows.Forms.MenuItem miPaste;
		private System.Windows.Forms.MenuItem miFile;
		private System.Windows.Forms.MenuItem miAbout;
		private System.Windows.Forms.MenuItem miView;
		private System.Windows.Forms.OpenFileDialog openFileDlg;
		private System.Windows.Forms.MenuItem miSerializeEntities;
		private System.Windows.Forms.MenuItem miDeserializeEntities;
		private System.Windows.Forms.SaveFileDialog dlgSaveSerializeFile;
		private System.Windows.Forms.OpenFileDialog dlgOpenSerializeFile;
		private System.Windows.Forms.MenuItem miOpenFile;
		private System.Windows.Forms.MenuItem miSaveFile;
		private System.Windows.Forms.MenuItem miSaveAsDXF;
		private System.Windows.Forms.MenuItem miPrint;
		private System.Windows.Forms.MenuItem miPrintPreview;
		private System.Windows.Forms.MenuItem miClose;
		private System.Windows.Forms.MenuItem miExit;
		private System.Windows.Forms.MenuItem miSep1;
		private System.Windows.Forms.MenuItem miCopyAsBMP;
		private System.Windows.Forms.MenuItem miSep2;
		private System.Windows.Forms.MenuItem miZoomIn;
		private System.Windows.Forms.MenuItem miSep3;
		private System.Windows.Forms.MenuItem miZoomOut;
		private System.Windows.Forms.MenuItem miEntitiesTree;
		private System.Windows.Forms.MenuItem miScale;
		private System.Windows.Forms.MenuItem miZoom10;
		private System.Windows.Forms.MenuItem miZoom25;
		private System.Windows.Forms.MenuItem miZoom50;
		private System.Windows.Forms.MenuItem miZoom100;
		private System.Windows.Forms.MenuItem miZoom200;
		private System.Windows.Forms.MenuItem miZoom400;
		private System.Windows.Forms.MenuItem miZoom800;
		private System.Windows.Forms.MenuItem miFit;
		private System.Windows.Forms.MenuItem miSHXFontsPaths;
		private System.Windows.Forms.MenuItem miOptions;
		private System.Windows.Forms.MenuItem miColorDraw;
		private System.Windows.Forms.MenuItem miBlackDraw;
		private System.Windows.Forms.MenuItem miWhiteBack;
		private System.Windows.Forms.MenuItem miBlackBack;
		private System.Windows.Forms.MenuItem miSep4;
		private System.Windows.Forms.MenuItem miShowLineWeight;
		private System.Windows.Forms.MenuItem miArcsSplit;
		private System.Windows.Forms.MenuItem miDimShow;
		private System.Windows.Forms.MenuItem miTextsShow;
		private System.Windows.Forms.MenuItem miSep5;
		private System.Windows.Forms.MenuItem miLayersShow;
		private System.Windows.Forms.MenuItem miRegister;
		private System.Windows.Forms.MenuItem miEdit;
		private System.Windows.Forms.MenuItem miCADFiles;
		private System.Windows.Forms.MenuItem miReg; 		
		private System.Windows.Forms.ToolBarButton tlbSnap;	

		#region Help
		#if RUSHELP
		/// <summary>
		///       
		/// </summary>
		#else
		/// <summary>
		/// Gets or sets scale of the drawing image
		/// </summary>
		#endif
		#endregion
		public float ImageScale
		{
			get
			{
				return this.sc;
			}
			set
			{
				this.sc = value;
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///        
		/// </summary>
		#else
		/// <summary>
		/// Gets or sets base dimensions of the image drawing area
		/// </summary>
		#endif
		#endregion
		public SizeF VisibleAreaSize
		{
			get
			{
				return this.visibleArea;
			}
			set
			{
				this.visibleArea = value;
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///       
		/// </summary>
		#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
		#endif
		#endregion
		public RectangleF ImageRectangleF
		{
			get
			{
				return new RectangleF(pos.X, pos.Y, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);
			}
		}

		#region Help
		/// <summary>
		/// A previous value of the scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImagePreviousScale
		{
			get
			{
				return this.prev_sc;
			}
			set
			{
				this.prev_sc = value;
			}
		}


		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public DRect ImageDRect
		{
			get
			{
				return new DRect(pos.X, pos.Y, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///      
		/// </summary>
		#else
		/// <summary>
		/// Gets or sets a position of the drawing image
		/// </summary>
		#endif
		#endregion
		public PointF Position
		{
			get
			{
				return this.pos;
			}
			set
			{
				this.pos = value;
			}
		}
		
		#region Help
		/// <summary>
		/// A previous position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>
		#endregion Help
		public PointF PreviousPosition
		{
			get
			{
				return this.old_Pos;
			}
			set
			{
				this.old_Pos = value;
			}
		}	

		#region Help
#if RUSHELP
		/// <summary>
		///         X
		/// </summary>
#else	
		/// <summary>
		/// Gets or sets a position of the drawing image by X axis
		/// </summary>
		
#endif
		#endregion
		public float LeftImagePosition
		{
			get
			{
				return this.pos.X;
			}
			set
			{
				this.pos.X = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         Y
		/// </summary>
#else
		/// <summary>
		/// Gets or sets a position of the drawing image by Y axis
		/// </summary>
#endif
		#endregion
		public float TopImagePosition
		{
			get
			{
				return this.pos.Y;
			}
			set
			{
				this.pos.Y = value;
			}
		}
		
		#region Help
		#if RUSHELP	
		/// <summary>
		///    <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>     
		/// </summary>
		#else
		/// <summary>
		/// Gets <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see> control, where an image is drawn
		/// </summary>
		#endif
		#endregion
		public CADImport.FaceModule.CADPictureBox EditorCADPictureBox
		{
			get
			{
				 return this.cadPictBox;
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///    (    )   
		/// </summary>
		#else
		/// <summary>
		/// Gets or sets entity point which was selected before offsetting or editing by marker
		/// </summary>
		#endif
		#endregion
		public Point StartPoint 
		{
			get
			{
				return this.startPoint;
			}
			set
			{
				this.startPoint = value;
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///   ,     (    )
		/// </summary>
		#else
		/// <summary>
		/// Gets or sets entity point which entity has after offsetting or editing by marker
		/// </summary>
		#endif
		#endregion
		public Point EndPoint 
		{
			get
			{
				return this.endPoint;
			}
			set
			{
				this.endPoint = value;
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///   <see cref="CADImport.CADImage"/>CADImage,
		///      / 
		///   cad 
		/// </summary>
		#else
	
		/// <summary>
		/// Gets or sets <see cref="CADImport.CADImage">CADImage</see> object where new entities are added
		/// </summary>
	
		#endif
		#endregion Help
		public CADImage Image
		{
			get
			{
				return this.cadImage;
			}
		}
		
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.stBar = new System.Windows.Forms.StatusBar();
			this.sbOpen = new System.Windows.Forms.StatusBarPanel();
			this.sbScale = new System.Windows.Forms.StatusBarPanel();
			this.sbCoord = new System.Windows.Forms.StatusBarPanel();
			this.tlbTool = new System.Windows.Forms.ToolBar();
			this.tlbOpen = new System.Windows.Forms.ToolBarButton();
			this.tlbZoomIn = new System.Windows.Forms.ToolBarButton();
			this.tlbZoomOut = new System.Windows.Forms.ToolBarButton();
			this.tlbWhite = new System.Windows.Forms.ToolBarButton();
			this.tlbBlack = new System.Windows.Forms.ToolBarButton();
			this.tlbLay = new System.Windows.Forms.ToolBarButton();
			this.tlbSave = new System.Windows.Forms.ToolBarButton();
			this.tlbShx = new System.Windows.Forms.ToolBarButton();
			this.tlbAddImgEnt = new System.Windows.Forms.ToolBarButton();
			this.tlbAddSolid = new System.Windows.Forms.ToolBarButton();
			this.tlbLine = new System.Windows.Forms.ToolBarButton();
			this.tlbRect = new System.Windows.Forms.ToolBarButton();
			this.tlbCircle = new System.Windows.Forms.ToolBarButton();
			this.tlbText = new System.Windows.Forms.ToolBarButton();
			this.tlbPoly = new System.Windows.Forms.ToolBarButton();
			this.tlbPen = new System.Windows.Forms.ToolBarButton();
			this.tlbAddHatch = new System.Windows.Forms.ToolBarButton();
			this.tlbCloud = new System.Windows.Forms.ToolBarButton();
			this.tlbAddMText = new System.Windows.Forms.ToolBarButton();
			this.tlbPoint = new System.Windows.Forms.ToolBarButton();
			this.tlbOptions = new System.Windows.Forms.ToolBarButton();
			this.tlbNew = new System.Windows.Forms.ToolBarButton();
			this.tlbGetExt = new System.Windows.Forms.ToolBarButton();
			this.tlbDel = new System.Windows.Forms.ToolBarButton();
			this.tlbUndo = new System.Windows.Forms.ToolBarButton();
			this.tlbRedo = new System.Windows.Forms.ToolBarButton();
			this.tlbSnap = new System.Windows.Forms.ToolBarButton();
			this.toolBtnImageList = new System.Windows.Forms.ImageList(this.components);
			this.cadPictBox = new CADImport.FaceModule.CADPictureBox();
			this.saveImgDlg = new System.Windows.Forms.SaveFileDialog();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.miFile = new System.Windows.Forms.MenuItem();
			this.miOpenFile = new System.Windows.Forms.MenuItem();
			this.miSaveFile = new System.Windows.Forms.MenuItem();
			this.miSaveAsDXF = new System.Windows.Forms.MenuItem();
			this.miSerializeEntities = new System.Windows.Forms.MenuItem();
			this.miDeserializeEntities = new System.Windows.Forms.MenuItem();
			this.miPrint = new System.Windows.Forms.MenuItem();
			this.miPrintCustom = new System.Windows.Forms.MenuItem();
			this.miPrintPreview = new System.Windows.Forms.MenuItem();
			this.miClose = new System.Windows.Forms.MenuItem();
			this.miSep1 = new System.Windows.Forms.MenuItem();
			this.miExit = new System.Windows.Forms.MenuItem();
			this.miEdit = new System.Windows.Forms.MenuItem();
			this.miCopyAsBMP = new System.Windows.Forms.MenuItem();
			this.miUndo = new System.Windows.Forms.MenuItem();
			this.miRedo = new System.Windows.Forms.MenuItem();
			this.miDelete = new System.Windows.Forms.MenuItem();
			this.miCopy = new System.Windows.Forms.MenuItem();
			this.miCut = new System.Windows.Forms.MenuItem();
			this.miPaste = new System.Windows.Forms.MenuItem();
			this.miView = new System.Windows.Forms.MenuItem();
			this.miEntitiesTree = new System.Windows.Forms.MenuItem();
			this.miSep2 = new System.Windows.Forms.MenuItem();
			this.miZoomIn = new System.Windows.Forms.MenuItem();
			this.miZoomOut = new System.Windows.Forms.MenuItem();
			this.miScale = new System.Windows.Forms.MenuItem();
			this.miZoom10 = new System.Windows.Forms.MenuItem();
			this.miZoom25 = new System.Windows.Forms.MenuItem();
			this.miZoom50 = new System.Windows.Forms.MenuItem();
			this.miZoom100 = new System.Windows.Forms.MenuItem();
			this.miZoom200 = new System.Windows.Forms.MenuItem();
			this.miZoom400 = new System.Windows.Forms.MenuItem();
			this.miZoom800 = new System.Windows.Forms.MenuItem();
			this.miFit = new System.Windows.Forms.MenuItem();
			this.miSep3 = new System.Windows.Forms.MenuItem();
			this.miSHXFontsPaths = new System.Windows.Forms.MenuItem();
			this.miOptions = new System.Windows.Forms.MenuItem();
			this.miCADFiles = new System.Windows.Forms.MenuItem();
			this.miColorDraw = new System.Windows.Forms.MenuItem();
			this.miBlackDraw = new System.Windows.Forms.MenuItem();
			this.miWhiteBack = new System.Windows.Forms.MenuItem();
			this.miBlackBack = new System.Windows.Forms.MenuItem();
			this.miSep4 = new System.Windows.Forms.MenuItem();
			this.miShowLineWeight = new System.Windows.Forms.MenuItem();
			this.miArcsSplit = new System.Windows.Forms.MenuItem();
			this.miDimShow = new System.Windows.Forms.MenuItem();
			this.miTextsShow = new System.Windows.Forms.MenuItem();
			this.miSep5 = new System.Windows.Forms.MenuItem();
			this.miLayersShow = new System.Windows.Forms.MenuItem();
			this.miReg = new System.Windows.Forms.MenuItem();
			this.miLng = new System.Windows.Forms.MenuItem();
			this.miRegister = new System.Windows.Forms.MenuItem();
			this.miFloatLic = new System.Windows.Forms.MenuItem();
			this.miAbout = new System.Windows.Forms.MenuItem();
			this.saveDXFDlg = new System.Windows.Forms.SaveFileDialog();
			this.trvPanel = new System.Windows.Forms.Panel();
			this.trvEntity = new System.Windows.Forms.TreeView();
			this.spProp = new System.Windows.Forms.Splitter();
			this.propGrid = new System.Windows.Forms.PropertyGrid();
			this.spltPictBox = new System.Windows.Forms.Splitter();
			this.printPrevDlg = new System.Windows.Forms.PrintPreviewDialog();
			this.setColorDlg = new System.Windows.Forms.ColorDialog();
			this.openFileDlg = new System.Windows.Forms.OpenFileDialog();
			this.dlgSaveSerializeFile = new System.Windows.Forms.SaveFileDialog();
			this.dlgOpenSerializeFile = new System.Windows.Forms.OpenFileDialog();
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).BeginInit();
			this.trvPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// stBar
			// 
			this.stBar.Font = new System.Drawing.Font("Microsoft Sans Serif", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.stBar.Location = new System.Drawing.Point(208, 382);
			this.stBar.Name = "stBar";
			this.stBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																					 this.sbOpen,
																					 this.sbScale,
																					 this.sbCoord});
			this.stBar.ShowPanels = true;
			this.stBar.Size = new System.Drawing.Size(416, 16);
			this.stBar.TabIndex = 9;
			// 
			// sbOpen
			// 
			this.sbOpen.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbOpen.MinWidth = 100;
			this.sbOpen.ToolTipText = "File Name";
			// 
			// sbScale
			// 
			this.sbScale.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbScale.Text = "100%";
			this.sbScale.ToolTipText = "Scale";
			this.sbScale.Width = 42;
			// 
			// sbCoord
			// 
			this.sbCoord.Alignment = System.Windows.Forms.HorizontalAlignment.Right;
			this.sbCoord.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbCoord.Text = "0 - 0 - 0";
			this.sbCoord.Width = 51;
			// 
			// tlbTool
			// 
			this.tlbTool.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.tlbTool.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.tlbTool.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																					   this.tlbOpen,
																					   this.tlbZoomIn,
																					   this.tlbZoomOut,
																					   this.tlbWhite,
																					   this.tlbBlack,
																					   this.tlbLay,
																					   this.tlbSave,
																					   this.tlbShx,
																					   this.tlbAddImgEnt,
																					   this.tlbAddSolid,
																					   this.tlbLine,
																					   this.tlbRect,
																					   this.tlbCircle,
																					   this.tlbText,
																					   this.tlbPoly,
																					   this.tlbPen,
																					   this.tlbAddHatch,
																					   this.tlbCloud,
																					   this.tlbAddMText,
																					   this.tlbPoint,
																					   this.tlbOptions,
																					   this.tlbNew,
																					   this.tlbGetExt,
																					   this.tlbDel,
																					   this.tlbUndo,
																					   this.tlbRedo,
																					   this.tlbSnap});
			this.tlbTool.ButtonSize = new System.Drawing.Size(20, 20);
			this.tlbTool.DropDownArrows = true;
			this.tlbTool.ImageList = this.toolBtnImageList;
			this.tlbTool.Location = new System.Drawing.Point(0, 0);
			this.tlbTool.Name = "tlbTool";
			this.tlbTool.ShowToolTips = true;
			this.tlbTool.Size = new System.Drawing.Size(624, 29);
			this.tlbTool.TabIndex = 10;
			this.tlbTool.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.tlbTool_ButtonClick);
			this.tlbTool.KeyDown += new System.Windows.Forms.KeyEventHandler(this.CADPictBoxKeyDown);
			// 
			// tlbOpen
			// 
			this.tlbOpen.ImageIndex = 1;
			this.tlbOpen.ToolTipText = "Open Picture";
			// 
			// tlbZoomIn
			// 
			this.tlbZoomIn.Enabled = false;
			this.tlbZoomIn.ImageIndex = 2;
			this.tlbZoomIn.ToolTipText = "Zoom In (+)";
			// 
			// tlbZoomOut
			// 
			this.tlbZoomOut.Enabled = false;
			this.tlbZoomOut.ImageIndex = 3;
			this.tlbZoomOut.ToolTipText = "Zoom Out (-)";
			// 
			// tlbWhite
			// 
			this.tlbWhite.Enabled = false;
			this.tlbWhite.ImageIndex = 4;
			this.tlbWhite.ToolTipText = "White Background";
			// 
			// tlbBlack
			// 
			this.tlbBlack.Enabled = false;
			this.tlbBlack.ImageIndex = 5;
			this.tlbBlack.ToolTipText = "Black Background";
			// 
			// tlbLay
			// 
			this.tlbLay.Enabled = false;
			this.tlbLay.ImageIndex = 6;
			this.tlbLay.ToolTipText = "Show Layers";
			// 
			// tlbSave
			// 
			this.tlbSave.Enabled = false;
			this.tlbSave.ImageIndex = 7;
			this.tlbSave.ToolTipText = "Save Picture...";
			// 
			// tlbShx
			// 
			this.tlbShx.Enabled = false;
			this.tlbShx.ImageIndex = 8;
			this.tlbShx.Pushed = true;
			this.tlbShx.ToolTipText = "Use SHX fonts";
			// 
			// tlbAddImgEnt
			// 
			this.tlbAddImgEnt.ImageIndex = 12;
			this.tlbAddImgEnt.ToolTipText = "Add new ImageEnt";
			// 
			// tlbAddSolid
			// 
			this.tlbAddSolid.ImageIndex = 10;
			this.tlbAddSolid.ToolTipText = "Add new solid";
			// 
			// tlbLine
			// 
			this.tlbLine.ImageIndex = 13;
			this.tlbLine.ToolTipText = "Add new line";
			// 
			// tlbRect
			// 
			this.tlbRect.ImageIndex = 14;
			this.tlbRect.ToolTipText = "Add new rect";
			// 
			// tlbCircle
			// 
			this.tlbCircle.ImageIndex = 15;
			this.tlbCircle.ToolTipText = "Add new ellipse";
			// 
			// tlbText
			// 
			this.tlbText.ImageIndex = 16;
			this.tlbText.ToolTipText = "Add new text";
			// 
			// tlbPoly
			// 
			this.tlbPoly.ImageIndex = 17;
			this.tlbPoly.ToolTipText = "Add new poly";
			// 
			// tlbPen
			// 
			this.tlbPen.ImageIndex = 18;
			this.tlbPen.ToolTipText = "Add new pen";
			// 
			// tlbAddHatch
			// 
			this.tlbAddHatch.ImageIndex = 26;
			// 
			// tlbCloud
			// 
			this.tlbCloud.ImageIndex = 25;
			// 
			// tlbAddMText
			// 
			this.tlbAddMText.ImageIndex = 16;
			this.tlbAddMText.ToolTipText = "Add MText";
			// 
			// tlbPoint
			// 
			this.tlbPoint.ImageIndex = 21;
			// 
			// tlbOptions
			// 
			this.tlbOptions.ImageIndex = 9;
			this.tlbOptions.ToolTipText = "Options";
			// 
			// tlbNew
			// 
			this.tlbNew.ImageIndex = 11;
			this.tlbNew.ToolTipText = "New Picture";
			// 
			// tlbGetExt
			// 
			this.tlbGetExt.ImageIndex = 20;
			this.tlbGetExt.ToolTipText = "Fit drawing to window";
			// 
			// tlbDel
			// 
			this.tlbDel.Enabled = false;
			this.tlbDel.ImageIndex = 24;
			// 
			// tlbUndo
			// 
			this.tlbUndo.Enabled = false;
			this.tlbUndo.ImageIndex = 23;
			this.tlbUndo.ToolTipText = "Undo";
			// 
			// tlbRedo
			// 
			this.tlbRedo.Enabled = false;
			this.tlbRedo.ImageIndex = 22;
			// 
			// tlbSnap
			// 
			this.tlbSnap.ImageIndex = 27;
			this.tlbSnap.Pushed = true;
			// 
			// toolBtnImageList
			// 
			this.toolBtnImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
			this.toolBtnImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.toolBtnImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("toolBtnImageList.ImageStream")));
			this.toolBtnImageList.TransparentColor = System.Drawing.Color.Fuchsia;
			// 
			// cadPictBox
			// 
			this.cadPictBox.BackColor = System.Drawing.Color.Black;
			this.cadPictBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.cadPictBox.Cursor = System.Windows.Forms.Cursors.Default;
			this.cadPictBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.cadPictBox.DoubleBuffering = true;
			this.cadPictBox.Location = new System.Drawing.Point(208, 29);
			this.cadPictBox.Name = "cadPictBox";
			this.cadPictBox.Position = new System.Drawing.Point(0, 0);
			this.cadPictBox.Size = new System.Drawing.Size(416, 353);
			this.cadPictBox.TabIndex = 13;
			this.cadPictBox.TabStop = false;
			this.cadPictBox.VirtualSize = new System.Drawing.Size(0, 0);
			this.cadPictBox.MouseUp += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseUp);
			this.cadPictBox.Paint += new System.Windows.Forms.PaintEventHandler(this.cadPictBox_Paint);
			this.cadPictBox.MouseMove += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseMove);
			this.cadPictBox.MouseDown += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseDown);
			// 
			// saveImgDlg
			// 
			this.saveImgDlg.Filter = "*.bmp|*.bmp|*.jpg|*.jpg|*.tiff|*.tiff|*.gif|*.gif|*.emf|*.emf|*.dxf|*.dxf";
			this.saveImgDlg.RestoreDirectory = true;
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.miFile,
																					 this.miEdit,
																					 this.miView,
																					 this.miCADFiles,
																					 this.miReg});
			// 
			// miFile
			// 
			this.miFile.Index = 0;
			this.miFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miOpenFile,
																				   this.miSaveFile,
																				   this.miSaveAsDXF,
																				   this.miSerializeEntities,
																				   this.miDeserializeEntities,
																				   this.miPrint,
																				   this.miPrintCustom,
																				   this.miPrintPreview,
																				   this.miClose,
																				   this.miSep1,
																				   this.miExit});
			this.miFile.Text = "&File";
			// 
			// miOpenFile
			// 
			this.miOpenFile.Index = 0;
			this.miOpenFile.Text = "Open Picture...";
			this.miOpenFile.Click += new System.EventHandler(this.miOpenFile_Click);
			// 
			// miSaveFile
			// 
			this.miSaveFile.Enabled = false;
			this.miSaveFile.Index = 1;
			this.miSaveFile.Text = "Save Picture...";
			this.miSaveFile.Click += new System.EventHandler(this.miSaveFile_Click);
			// 
			// miSaveAsDXF
			// 
			this.miSaveAsDXF.Enabled = false;
			this.miSaveAsDXF.Index = 2;
			this.miSaveAsDXF.Text = "Save as DXF...";
			this.miSaveAsDXF.Click += new System.EventHandler(this.miSaveAsDXF_Click);
			// 
			// miSerializeEntities
			// 
			this.miSerializeEntities.Enabled = false;
			this.miSerializeEntities.Index = 3;
			this.miSerializeEntities.Text = "Serialize entities...";
			this.miSerializeEntities.Click += new System.EventHandler(this.miSerializeEntities_Click);
			// 
			// miDeserializeEntities
			// 
			this.miDeserializeEntities.Index = 4;
			this.miDeserializeEntities.Text = "Deserialize entities...";
			this.miDeserializeEntities.Click += new System.EventHandler(this.miDeserializeEntities_Click);
			// 
			// miPrint
			// 
			this.miPrint.Enabled = false;
			this.miPrint.Index = 5;
			this.miPrint.Text = "Print ";
			this.miPrint.Click += new System.EventHandler(this.miPrint_Click);
			// 
			// miPrintCustom
			// 
			this.miPrintCustom.Enabled = false;
			this.miPrintCustom.Index = 6;
			this.miPrintCustom.Text = "Custom Print Preview...";
			this.miPrintCustom.Click += new System.EventHandler(this.miPrintCustom_Click);
			// 
			// miPrintPreview
			// 
			this.miPrintPreview.Enabled = false;
			this.miPrintPreview.Index = 7;
			this.miPrintPreview.Text = "Print Preview...";
			this.miPrintPreview.Click += new System.EventHandler(this.miPrintPreview_Click);
			// 
			// miClose
			// 
			this.miClose.Enabled = false;
			this.miClose.Index = 8;
			this.miClose.Text = "Close Picture";
			this.miClose.Click += new System.EventHandler(this.miClose_Click);
			// 
			// miSep1
			// 
			this.miSep1.Index = 9;
			this.miSep1.Text = "-";
			// 
			// miExit
			// 
			this.miExit.Index = 10;
			this.miExit.Text = "Close";
			this.miExit.Click += new System.EventHandler(this.miExit_Click);
			// 
			// miEdit
			// 
			this.miEdit.Index = 1;
			this.miEdit.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miCopyAsBMP,
																				   this.miUndo,
																				   this.miRedo,
																				   this.miDelete,
																				   this.miCopy,
																				   this.miCut,
																				   this.miPaste});
			this.miEdit.Text = "&Edit";
			// 
			// miCopyAsBMP
			// 
			this.miCopyAsBMP.Enabled = false;
			this.miCopyAsBMP.Index = 0;
			this.miCopyAsBMP.Text = "Copy as BMP";
			this.miCopyAsBMP.Click += new System.EventHandler(this.miCopyAsBMP_Click);
			// 
			// miUndo
			// 
			this.miUndo.Enabled = false;
			this.miUndo.Index = 1;
			this.miUndo.Shortcut = System.Windows.Forms.Shortcut.CtrlZ;
			this.miUndo.Text = "Undo";
			this.miUndo.Click += new System.EventHandler(this.miUndo_Click);
			// 
			// miRedo
			// 
			this.miRedo.Enabled = false;
			this.miRedo.Index = 2;
			this.miRedo.Shortcut = System.Windows.Forms.Shortcut.CtrlY;
			this.miRedo.Text = "Redo";
			this.miRedo.Click += new System.EventHandler(this.miRedo_Click);
			// 
			// miDelete
			// 
			this.miDelete.Enabled = false;
			this.miDelete.Index = 3;
			this.miDelete.Shortcut = System.Windows.Forms.Shortcut.Del;
			this.miDelete.Text = "Delete";
			this.miDelete.Click += new System.EventHandler(this.miDelete_Click);
			// 
			// miCopy
			// 
			this.miCopy.Enabled = false;
			this.miCopy.Index = 4;
			this.miCopy.Shortcut = System.Windows.Forms.Shortcut.CtrlC;
			this.miCopy.Text = "&Copy";
			this.miCopy.Click += new System.EventHandler(this.miCopy_Click);
			// 
			// miCut
			// 
			this.miCut.Enabled = false;
			this.miCut.Index = 5;
			this.miCut.Shortcut = System.Windows.Forms.Shortcut.CtrlX;
			this.miCut.Text = "Cu&t";
			this.miCut.Click += new System.EventHandler(this.miCut_Click);
			// 
			// miPaste
			// 
			this.miPaste.Enabled = false;
			this.miPaste.Index = 6;
			this.miPaste.Shortcut = System.Windows.Forms.Shortcut.CtrlV;
			this.miPaste.Text = "Paste";
			this.miPaste.Click += new System.EventHandler(this.miPaste_Click);
			// 
			// miView
			// 
			this.miView.Index = 2;
			this.miView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miEntitiesTree,
																				   this.miSep2,
																				   this.miZoomIn,
																				   this.miZoomOut,
																				   this.miScale,
																				   this.miFit,
																				   this.miSep3,
																				   this.miSHXFontsPaths,
																				   this.miOptions});
			this.miView.Text = "&View";
			// 
			// miEntitiesTree
			// 
			this.miEntitiesTree.Checked = true;
			this.miEntitiesTree.Index = 0;
			this.miEntitiesTree.Text = "Show entities panel";
			this.miEntitiesTree.Click += new System.EventHandler(this.miEntitiesTree_Click);
			// 
			// miSep2
			// 
			this.miSep2.Index = 1;
			this.miSep2.Text = "-";
			// 
			// miZoomIn
			// 
			this.miZoomIn.Enabled = false;
			this.miZoomIn.Index = 2;
			this.miZoomIn.Text = "Zoom In (+)";
			this.miZoomIn.Click += new System.EventHandler(this.miZoomIn_Click);
			// 
			// miZoomOut
			// 
			this.miZoomOut.Enabled = false;
			this.miZoomOut.Index = 3;
			this.miZoomOut.Text = "Zoom Out (-)";
			this.miZoomOut.Click += new System.EventHandler(this.miZoomOut_Click);
			// 
			// miScale
			// 
			this.miScale.Enabled = false;
			this.miScale.Index = 4;
			this.miScale.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.miZoom10,
																					this.miZoom25,
																					this.miZoom50,
																					this.miZoom100,
																					this.miZoom200,
																					this.miZoom400,
																					this.miZoom800});
			this.miScale.Text = "Scale";
			// 
			// miZoom10
			// 
			this.miZoom10.Index = 0;
			this.miZoom10.Text = "10%";
			this.miZoom10.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom25
			// 
			this.miZoom25.Index = 1;
			this.miZoom25.Text = "25%";
			this.miZoom25.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom50
			// 
			this.miZoom50.Index = 2;
			this.miZoom50.Text = "50%";
			this.miZoom50.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom100
			// 
			this.miZoom100.Index = 3;
			this.miZoom100.Text = "100%";
			this.miZoom100.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom200
			// 
			this.miZoom200.Index = 4;
			this.miZoom200.Text = "200%";
			this.miZoom200.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom400
			// 
			this.miZoom400.Index = 5;
			this.miZoom400.Text = "400%";
			this.miZoom400.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom800
			// 
			this.miZoom800.Index = 6;
			this.miZoom800.Text = "800%";
			this.miZoom800.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miFit
			// 
			this.miFit.Enabled = false;
			this.miFit.Index = 5;
			this.miFit.Text = "Fit drawing to window";
			this.miFit.Click += new System.EventHandler(this.miFit_Click);
			// 
			// miSep3
			// 
			this.miSep3.Index = 6;
			this.miSep3.Text = "-";
			// 
			// miSHXFontsPaths
			// 
			this.miSHXFontsPaths.Index = 7;
			this.miSHXFontsPaths.Text = "SHX Fonts";
			this.miSHXFontsPaths.Click += new System.EventHandler(this.miSHXFontsPaths_Click);
			// 
			// miOptions
			// 
			this.miOptions.Index = 8;
			this.miOptions.Text = "Options";
			this.miOptions.Click += new System.EventHandler(this.miOptions_Click);
			// 
			// miCADFiles
			// 
			this.miCADFiles.Index = 3;
			this.miCADFiles.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					   this.miColorDraw,
																					   this.miBlackDraw,
																					   this.miWhiteBack,
																					   this.miBlackBack,
																					   this.miSep4,
																					   this.miShowLineWeight,
																					   this.miArcsSplit,
																					   this.miDimShow,
																					   this.miTextsShow,
																					   this.miSep5,
																					   this.miLayersShow});
			this.miCADFiles.Text = "&CAD files";
			// 
			// miColorDraw
			// 
			this.miColorDraw.Checked = true;
			this.miColorDraw.Enabled = false;
			this.miColorDraw.Index = 0;
			this.miColorDraw.RadioCheck = true;
			this.miColorDraw.Text = "Color drawing";
			this.miColorDraw.Click += new System.EventHandler(this.miColorDraw_Click);
			// 
			// miBlackDraw
			// 
			this.miBlackDraw.Enabled = false;
			this.miBlackDraw.Index = 1;
			this.miBlackDraw.RadioCheck = true;
			this.miBlackDraw.Text = "Black drawing";
			this.miBlackDraw.Click += new System.EventHandler(this.miBlackDraw_Click);
			// 
			// miWhiteBack
			// 
			this.miWhiteBack.Enabled = false;
			this.miWhiteBack.Index = 2;
			this.miWhiteBack.RadioCheck = true;
			this.miWhiteBack.Text = "White Background";
			this.miWhiteBack.Click += new System.EventHandler(this.miWhiteBack_Click);
			// 
			// miBlackBack
			// 
			this.miBlackBack.Enabled = false;
			this.miBlackBack.Index = 3;
			this.miBlackBack.RadioCheck = true;
			this.miBlackBack.Text = "Black Background";
			this.miBlackBack.Click += new System.EventHandler(this.miBlackBack_Click);
			// 
			// miSep4
			// 
			this.miSep4.Enabled = false;
			this.miSep4.Index = 4;
			this.miSep4.Text = "-";
			// 
			// miShowLineWeight
			// 
			this.miShowLineWeight.Checked = true;
			this.miShowLineWeight.Enabled = false;
			this.miShowLineWeight.Index = 5;
			this.miShowLineWeight.Text = "Show Lineweight";
			this.miShowLineWeight.Click += new System.EventHandler(this.miShowLineWeight_Click);
			// 
			// miArcsSplit
			// 
			this.miArcsSplit.Checked = true;
			this.miArcsSplit.Enabled = false;
			this.miArcsSplit.Index = 6;
			this.miArcsSplit.Text = "Arcs Splitted";
			this.miArcsSplit.Click += new System.EventHandler(this.miArcsSplit_Click);
			// 
			// miDimShow
			// 
			this.miDimShow.Checked = true;
			this.miDimShow.Enabled = false;
			this.miDimShow.Index = 7;
			this.miDimShow.Text = "Dimensions Show";
			this.miDimShow.Click += new System.EventHandler(this.miDimShow_Click);
			// 
			// miTextsShow
			// 
			this.miTextsShow.Checked = true;
			this.miTextsShow.Enabled = false;
			this.miTextsShow.Index = 8;
			this.miTextsShow.Text = "Texts Show";
			this.miTextsShow.Click += new System.EventHandler(this.miTextsShow_Click);
			// 
			// miSep5
			// 
			this.miSep5.Enabled = false;
			this.miSep5.Index = 9;
			this.miSep5.Text = "-";
			// 
			// miLayersShow
			// 
			this.miLayersShow.Enabled = false;
			this.miLayersShow.Index = 10;
			this.miLayersShow.Text = "Show Layers";
			this.miLayersShow.Click += new System.EventHandler(this.miLayersShow_Click);
			// 
			// miReg
			// 
			this.miReg.Index = 4;
			this.miReg.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				  this.miLng,
																				  this.miRegister,
																				  this.miFloatLic,
																				  this.miAbout});
			this.miReg.Text = "?";
			// 
			// miLng
			// 
			this.miLng.Index = 0;
			this.miLng.Text = "Language";
			// 
			// miRegister
			// 
			this.miRegister.Index = 1;
			this.miRegister.Text = "Register ";
			this.miRegister.Click += new System.EventHandler(this.miRegister_Click);
			// 
			// miFloatLic
			// 
			this.miFloatLic.Index = 2;
			this.miFloatLic.Text = "Floating License Registration...";
			this.miFloatLic.Click += new System.EventHandler(this.miFloatLic_Click);
			// 
			// miAbout
			// 
			this.miAbout.Index = 3;
			this.miAbout.Text = "About...";
			this.miAbout.Click += new System.EventHandler(this.miAbout_Click);
			// 
			// saveDXFDlg
			// 
			this.saveDXFDlg.DefaultExt = "*.dxf";
			this.saveDXFDlg.Filter = "*.dxf|*.dxf";
			// 
			// trvPanel
			// 
			this.trvPanel.Controls.Add(this.trvEntity);
			this.trvPanel.Controls.Add(this.spProp);
			this.trvPanel.Controls.Add(this.propGrid);
			this.trvPanel.Dock = System.Windows.Forms.DockStyle.Left;
			this.trvPanel.Location = new System.Drawing.Point(0, 29);
			this.trvPanel.Name = "trvPanel";
			this.trvPanel.Size = new System.Drawing.Size(208, 369);
			this.trvPanel.TabIndex = 14;
			// 
			// trvEntity
			// 
			this.trvEntity.BackColor = System.Drawing.Color.Black;
			this.trvEntity.BorderStyle = System.Windows.Forms.BorderStyle.None;
			this.trvEntity.CheckBoxes = true;
			this.trvEntity.Dock = System.Windows.Forms.DockStyle.Fill;
			this.trvEntity.ForeColor = System.Drawing.Color.White;
			this.trvEntity.ImageIndex = -1;
			this.trvEntity.Indent = 19;
			this.trvEntity.ItemHeight = 16;
			this.trvEntity.Location = new System.Drawing.Point(0, 0);
			this.trvEntity.Name = "trvEntity";
			this.trvEntity.SelectedImageIndex = -1;
			this.trvEntity.Size = new System.Drawing.Size(208, 118);
			this.trvEntity.TabIndex = 16;
			this.trvEntity.AfterCheck += new System.Windows.Forms.TreeViewEventHandler(this.trvEntity_AfterCheck);
			this.trvEntity.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.trvEntity_AfterSelect);
			// 
			// spProp
			// 
			this.spProp.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.spProp.Location = new System.Drawing.Point(0, 118);
			this.spProp.Name = "spProp";
			this.spProp.Size = new System.Drawing.Size(208, 3);
			this.spProp.TabIndex = 17;
			this.spProp.TabStop = false;
			// 
			// propGrid
			// 
			this.propGrid.CommandsVisibleIfAvailable = true;
			this.propGrid.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.propGrid.LargeButtons = false;
			this.propGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propGrid.Location = new System.Drawing.Point(0, 121);
			this.propGrid.Name = "propGrid";
			this.propGrid.Size = new System.Drawing.Size(208, 248);
			this.propGrid.TabIndex = 15;
			this.propGrid.Text = "propertyGrid1";
			this.propGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			this.propGrid.PropertyValueChanged += new System.Windows.Forms.PropertyValueChangedEventHandler(this.propGrid_PropertyValueChanged);
			// 
			// spltPictBox
			// 
			this.spltPictBox.Location = new System.Drawing.Point(208, 29);
			this.spltPictBox.Name = "spltPictBox";
			this.spltPictBox.Size = new System.Drawing.Size(3, 353);
			this.spltPictBox.TabIndex = 15;
			this.spltPictBox.TabStop = false;
			// 
			// printPrevDlg
			// 
			this.printPrevDlg.AutoScrollMargin = new System.Drawing.Size(0, 0);
			this.printPrevDlg.AutoScrollMinSize = new System.Drawing.Size(0, 0);
			this.printPrevDlg.ClientSize = new System.Drawing.Size(400, 300);
			this.printPrevDlg.Enabled = true;
			this.printPrevDlg.Icon = ((System.Drawing.Icon)(resources.GetObject("printPrevDlg.Icon")));
			this.printPrevDlg.Location = new System.Drawing.Point(594, 17);
			this.printPrevDlg.MinimumSize = new System.Drawing.Size(375, 250);
			this.printPrevDlg.Name = "printPrevDlg";
			this.printPrevDlg.TransparencyKey = System.Drawing.Color.Empty;
			this.printPrevDlg.Visible = false;
			// 
			// openFileDlg
			// 
			this.openFileDlg.DefaultExt = "*.dxf;*.dwg";
			this.openFileDlg.Filter = @"All(*.dxf,*.dwg,*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl,*.cgm, *.gbr, *.bmp,*.jpg, *.jpeg,*.tiff, *.tif, *.gif, *.ico, *.cur, *.png, *.emf, *.wmf)|*.bmp;*.jpg;*.jpeg;*.tiff; *.tif; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf; *.wmf; *.cgm;*.gbr|CAD files(*.dxf, *.dwg)|*.dxf;*.dwg|All raster files (*.bmp,*.jpg, *.jpeg,*.tiff,*.tif, *.gif, *.ico, *.cur, *.png)|*.bmp;*.jpg;*.jpeg;*.tiff;*.tif; *.gif;*.ico;*.cur; *.png|Metafiles ( *.emf, *.wmf)| *.emf; *.wmf|DXF files(*.dxf)|*.dxf|DWG files(*.dwg)|*.dwg|BMP(*.bmp)|*.bmp|JPG(*.jpg, *.jpeg)| *.jpg; *.jpeg|TIFF(*.tiff, *.tif)|*.tiff;*.tif|GIF(*.gif)|*.gif|ICO(*.ico)|*.ico|PNG(*.png)|*.png|Cursors(*.cur)|*.cur|HPGL(*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl)|*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl|CGM (*.cgm)|*.cgm|Gerber (*.gbr)|*.gbr";
			this.openFileDlg.ShowHelp = true;
			// 
			// dlgSaveSerializeFile
			// 
			this.dlgSaveSerializeFile.Filter = "*.dat|*.dat";
			// 
			// dlgOpenSerializeFile
			// 
			this.dlgOpenSerializeFile.Filter = "*.dat|*.dat";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Control;
			this.ClientSize = new System.Drawing.Size(624, 398);
			this.Controls.Add(this.spltPictBox);
			this.Controls.Add(this.cadPictBox);
			this.Controls.Add(this.stBar);
			this.Controls.Add(this.trvPanel);
			this.Controls.Add(this.tlbTool);
			this.Menu = this.mainMenu;
			this.Name = "MainForm";
			this.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "CADImport.NET - Editor";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Resize += new System.EventHandler(this.MainForm_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.Deactivate += new System.EventHandler(this.MainForm_Deactivate);
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).EndInit();
			this.trvPanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		static MainForm()
		{					
			fileSettingsName = Application.StartupPath + @"\Settings.txt";			
		}

		[STAThread]
		static void Main(string[] args) 
		{
			Application.EnableVisualStyles();
			Application.DoEvents();						
			if(args.Length != 0)
				if(File.Exists(args[0]))
					fileName = args[0];
			Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(DoExceptionWorking);
			Application.Run(new MainForm());	
		}

		private static void DoExceptionWorking(object sender, System.Threading.ThreadExceptionEventArgs args)
		{				
			CADConst.SendExceptionReport(args.Exception);
		}

		private string RealScale
		{
			get
			{
				if(cadImage != null)
					return string.Format("{0,2:F}%", (((this.VisibleAreaSize.Width * ImageScale) / this.cadImage.AbsWidth) * cadImage.MMToPixelX) * 100.0);
				else 
					return string.Format("{0}%", ImageScale);
			}
		}

		private double RealScaleDouble
		{
			get
			{
				if(cadImage != null)
					return ((this.VisibleAreaSize.Width * ImageScale) / this.cadImage.AbsWidth) * cadImage.MMToPixelX;
				else 
					return ImageScale;
			}
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///    
		/// </summary>
		/// <param name="i">     </param>
		#else
		/// <summary>
		/// Adjusts the image size according to the specified scale.
		/// </summary>
		/// <param name="i">A scale value.</param>
		/// <remarks>If the scale value is more than one the image size increases; otherwise, decreases.</remarks>
		#endif
		#endregion Help
		public void Zoom(float i)
		{
			if(cadImage == null)
				return;
			ImageScale = ImageScale * i;
			if(ImageScale < 0.005f)
				ImageScale = 0.005f;	
			cadPictBox.Invalidate();						
			this.stopSnap = true;
			this.entCreator.Disable();
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///     ,         
		/// </summary>
		#else
		#endif
		#endregion
		private void Shift()
		{
			LeftImagePosition = PreviousPosition.X - (PreviousPosition.X - LeftImagePosition) * ImageScale / ImagePreviousScale;
			TopImagePosition = PreviousPosition.Y - (PreviousPosition.Y - TopImagePosition) * ImageScale / ImagePreviousScale;
			ImagePreviousScale = ImageScale;		
		}

		private void InitParams()
		{
			ImageScale = 1.0f;
			ImagePreviousScale = 1.0f;
			colorDraw = true;
			dimVisible = true;
			showLineWeight = true;
			textVisible = true;
			cntrlDown = false;
			startPoint = Point.Empty;
			endPoint = Point.Empty;
			PreviousPosition = PointF.Empty;
		}

		private void InitialCADImportForms()
		{
			lForm = new LayerForm();
			aboutForm = new AboutForm();
			prtForm = new CADImport.Printing.PrintingForm();			
			prtForm.LayerForm = lForm;
			shxFrm = new SHXForm();
			rasterSizeForm = new SetRasterSizeForm();
			#region protect
#if (protect && floatprotect)
			floatLicFrm = new FloatLicForm();
			floatLicFrm.Change += new System.EventHandler(InvalidateImage);
#endif
#if protect
			regFrm = new RegForm();
#endif
			#endregion protect
		}

		private void InitParamsState()
		{
			#region protect
#if ((! floatprotect) && protect)
			this.miFloatLic.Enabled = false;
#endif
			#endregion
			this.clipRectangle = new ClipRect(this.cadPictBox);
			this.clipRectangle.MultySelect = true;
			SetAddEntityMode(CreatorType.Undetected);
			this.entCreator = new EntitiesCreator(this.cadPictBox, Color.White);		
			CADImportFace.EntityPropertyGrid = this.propGrid;			
			lngFile = ApplicationConstants.defaultstr;
			curLngInd = 0;
			listItemSelectDel = new ListItemSelectEventHandler(ListSelect);			
			this.cadPictBox.MouseWheel += new MouseEventHandler(CADPictBoxMouseWeel);
			this.cadPictBox.ScrollEvent += new CADImport.FaceModule.ScrollEventHandlerExt(CADPictBoxScroll);
			this.cadPictBox.KeyDown += new System.Windows.Forms.KeyEventHandler(CADPictBoxKeyDown);				
			this.cadPictBox.KeyUp += new System.Windows.Forms.KeyEventHandler(CADPictBoxKeyUp);				
			addEntity = -1;		
			enableSnap = true;
			this.entCreator.EndEntityDraw += new CADImport.CADImportForms.ChangeOptionsEventHandler(this.EndAddEntity); 			
			this.entCreator.CreateNewCADImageEvent += new CADImport.Professional.CreateNewObjectEventHandler(this.CreateNewImage); 						
			this.entCreator.GetRealPointEvent += new  CADImport.Professional.GetRealPointEvent(this.GetRealPoint); 									
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///      
		/// </summary>
		#else
		/// <summary>
		/// Initializes a new instance of the <see cref="CADImportNetDemos.CADEditorDemo.MainForm">
		/// CADImportNetDemos.CADEditorDemo.MainForm</see> class.
		/// </summary>
		#endif	
		#endregion Help
		public MainForm()
		{		
			InitParams();			
			InitialCADImportForms();
			InitializeComponent();
			InitParamsState();											
			InitLng();
			InitSettings();
			SetAllFormsSettings();			
		}	
		#region Help
		#if RUSHELP
		/// <summary>
		///     
		/// </summary>
		/// <param name="val"> </param>
		/// <param name="index">    </param>
		#else
		/// <summary>
		/// Sets specified linguage for the application
		/// </summary>
		/// <param name="val">A language to set</param>
		/// <param name="index">An index of a language in the languages list</param>
		#endif
		#endregion
		public void ChangeLanguage(string val, int index)
		{
			curLngInd = index;
			lngFile = val + ApplicationConstants.lngextstr; 
			SetAllFormsLanguageSettings();
			SetLNG();
			this.ChangeControlState();
		}

		private void CADPictBoxKeyUp(object sender, KeyEventArgs e)
		{
			if((e.KeyCode == Keys.ControlKey)||(e.KeyCode == Keys.ShiftKey))
				this.cntrlDown = false;
		}
		#region Help
#if RUSHELP
		/// <summary>
		///      
		/// </summary>
#else
#endif
		#endregion
		private void SetAllFormsSettings()
		{	
			this.SetAllFormsLanguageSettings();				
			this.entCreator.OptionsForm.ChangeLngPath += new CADImport.CADImportForms.ChangeLngPathEventHandler(ReloadLNG);
			this.entCreator.OptionsForm.ChangeLanguage += new CADImport.CADImportForms.ChangeLanguageEventHandler(ChangeLanguage);						
		}		
					
		private void CADPictBoxKeyDown(object sender, KeyEventArgs e)
		{		
			if(!this.cadPictBox.Focused)
				this.cadPictBox.Focus();	
			int cn = 2;
			if(e.Shift)
				cn = 5;
			switch(e.KeyCode)
			{					
				case Keys.Up:	
					MoveEntity(0, cn, 0, cn);																				
					break;
				case Keys.Down:
					MoveEntity(0, -cn, 0, -cn);
					break;
				case Keys.Left:
					MoveEntity(cn, 0, cn, 0);
					break;
				case Keys.Right:
					MoveEntity(-cn, 0, -cn, 0);
					break;
				case Keys.Escape:
					ClearSelection();
					break;								
				case Keys.Enter:					
					this.entCreator.EndEntity();
					break;			
				case Keys.ControlKey:	
				case Keys.ShiftKey:
					this.cntrlDown = true;
					break;
			}											
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///      
		/// </summary>
		/// <param name="x1"> x </param>
		/// <param name="y1"> y </param>
		/// <param name="x2"> x </param>
		/// <param name="y2"> y </param>
		#else
		/// <summary>
		/// Moves selected entities to the specified position
		/// </summary>
		/// <param name="x1">Start X position</param>
		/// <param name="y1">Start Y position</param>
		/// <param name="x2">End X position</param>
		/// <param name="y2">End Y position</param>
		#endif
		#endregion
		public void MoveEntity(int x1, int y1, int x2, int y2)
		{
			this.endPoint.X = x1;
			this.endPoint.Y = y1;
			this.startPoint.X = x2;
			this.startPoint.Y = y2;
			SetNewEntityPosition();				
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///     
		/// </summary>
		#else
		/// <summary>
		/// Clears selection and redraws entities
		/// </summary>
		#endif
		#endregion
		public void ClearSelection()
		{
			this.entCreator.Disable();
			this.stopSnap = true;
			if(this.cadImage == null) 
				return;
			this.cadImage.ClearSelection();
			this.cadImage.ClearMarkers();
			this.propGrid.SelectedObject = null;						
			this.cadPictBox.Invalidate();			
		}

		private void CADPictBoxMouseWeel(object sender, System.Windows.Forms.MouseEventArgs e)
		{			
			if(e.Delta < 0)	
				Zoom(0.7f);
			else 
				Zoom(1.3f);
			this.Shift();
			SetPictureBoxPosition(Position);
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///        
		/// </summary>
		/// <param name="sender">  </param>
		/// <param name="e"> </param>
		#else
		/// <summary>
		/// Scrolls a drawing on scroll bar event appearance
		/// </summary>
		/// <param name="sender">An object which has invoked the event</param>
		/// <param name="e">Event parameters</param>
		#endif
		#endregion
		public void CADPictBoxScroll(object sender, CADImport.FaceModule.ScrollEventArgsExt e)
		{			
			LeftImagePosition = -this.cadPictBox.Position.X;
			TopImagePosition = -this.cadPictBox.Position.Y;
			this.cadPictBox.Invalidate();
		}
		#region Help
		/// <summary>
		///      
		/// </summary>
		/// <param name="sender">  </param>
		/// <param name="e"> </param>
		#endregion
		public void LanguageSelect_Click(object sender, System.EventArgs e)
		{
			if(!(sender is MenuItem)) 
				return;
			MenuItem mn = (sender as MenuItem);
			curLngInd = (byte)mn.Index;
			lngFile = mn.Text + ApplicationConstants.lngextstr; 
			SetAllFormsLanguageSettings();
			SetLNG();
			this.ChangeControlState();		
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///      lng 
		/// </summary>
		#else
		/// <summary>
		/// Sets a language by the current lng file
		/// </summary>
		#endif
		#endregion
		public void SetLNG()
		{
			mlng.LoadLNG(lngFile);
			mlng.RestoreLanguage(this.Controls, this.Menu);
			this.mainMenu.MenuItems[4].MenuItems[0].MenuItems.Clear();
			mlng.LoadLngFileList(this.mainMenu.MenuItems[4].MenuItems[0], new System.EventHandler(LanguageSelect_Click));
			this.Text = ApplicationConstants.appnamestr;
			this.Text = mlng.SetLanguage(this.Controls, this.Menu, this.Text);
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///   <see cref="MainForm.settingsLst"> </see>    
		/// </summary>
		#else
		/// <summary>
		/// Sets current properties for controls by <see cref="MainForm.settingsLst">properties list</see>
		/// </summary>
		#endif
		#endregion		
		public void SetSettings()
		{
			if(settingsLst == null) return;
			string tmp;
			int cn = 0;
			//Language path
			string key = ApplicationConstants.languagepath;
			if(settingsLst.ContainsKey(key))
			{
				this.mlng.Path = Convert.ToString(settingsLst[key]);
				this.entCreator.OptionsForm.LngDir = this.mlng.Path;
			}
			//Language
			key = ApplicationConstants.languagestr;
			if(settingsLst.ContainsKey(key))
				lngFile = (string)settingsLst[key];
			mlng.LoadLNG(lngFile);
			this.Text = mlng.SetLanguage(this.Controls, this.Menu, this.Text);
			//Language ID
			key = ApplicationConstants.languageIDstr;
			if(settingsLst.ContainsKey(key))
				this.curLngInd = Convert.ToByte(settingsLst[key]);
			//BackgroundColor
			key = ApplicationConstants.backcolorstr;
			tmp = ApplicationConstants.blackstr;
			if(settingsLst.ContainsKey(key))
				tmp = Convert.ToString(settingsLst[key]);
			if(tmp.ToUpper() == ApplicationConstants.blackstr)
				this.Black_Click();
			else 
				this.White_Click();
			//Show entity panel
			key = ApplicationConstants.showentitystr;
			if(settingsLst.ContainsKey(key))
				tmp = Convert.ToString(settingsLst[key]);
			if(tmp.ToUpper() == ApplicationConstants.truestr) 
			{
				this.spltPictBox.Visible = true;
				this.trvPanel.Visible = true;
			}
			else
			{
				this.spltPictBox.Visible = false;
				this.trvPanel.Visible = false;
			}
			//Color drawing
			key = ApplicationConstants.colordrawstr;
			if(settingsLst.ContainsKey(key))
				tmp = Convert.ToString(settingsLst[key]);
			if(tmp.ToUpper() == ApplicationConstants.truestr) 
				this.colorDraw = true;
			else this.colorDraw = false;
			//SHXPathCount
			key = ApplicationConstants.shxpathcnstr;
			if(settingsLst.ContainsKey(key))
				cn = Convert.ToInt32(settingsLst[key]);
			//SHXPaths
			for(int i = 0; i < cn; i++)
			{
				key = string.Format("SHXPath_{0}", (i + 1));
				if(settingsLst.ContainsKey(key))
					this.shxFrm.AddPath(Convert.ToString(settingsLst[key]));
			}
			//First start
			key = ApplicationConstants.installstr;
			if(settingsLst.ContainsKey(key))
			{
				if(CADConst.SearchSHXPaths)
				{
					this.shxFrm.lstDir.Items.Clear();
					this.shxFrm.lstPath.Clear();
					ArrayList vPaths = new ArrayList();
					CADConst.FindAutoCADSHXPaths(vPaths);
					for(int i = 0; i < vPaths.Count; i++)
					{
						tmp = (string)vPaths[i];
						this.shxFrm.lstDir.Items.Add(tmp);
						this.shxFrm.lstPath.Add(tmp, string.Empty);
						CADConst.SHXSearchPaths += CADConst.SHXSearchPaths + tmp + ApplicationConstants.sepstr3;
					}
				}
			}
			#region protect
#if protect
			//License
			key = "Type_license";
			if(settingsLst.ContainsKey(key))
			{
				tmp = (string)settingsLst[key];
#if floatprotect
				if(tmp == "floating")
				{
					Protection.LicenseType = LicenseType.Floating;
					key = "Host";
					if(settingsLst.ContainsKey(key))
					{
						this.floatLicFrm.Host = (string)settingsLst[key];
					}
					key = "Port";
					if(settingsLst.ContainsKey(key))
					{
						this.floatLicFrm.Port = Convert.ToInt32(settingsLst[key]);
					}
					key = "Application";
					if(settingsLst.ContainsKey(key))
					{
						this.floatLicFrm.App = (string)settingsLst[key];
					}
				}
				else
				{
					Protection.LicenseType = LicenseType.Register;
				}
#else
					Protection.LicenseType = LicenseType.Register;
#endif
			}
#endif
			#endregion
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///      <see cref="MainForm.settingsLst"> </see>
		/// </summary>
		#else
		/// <summary>
		/// Creates <see cref="MainForm.settingsLst">properties list</see> by current position of the controls
		/// </summary>
		#endif
		#endregion
		public void CreateNewSettingsList()
		{
			string tmp;
			settingsLst = new SortedList();
			//Language path
			settingsLst.Add(ApplicationConstants.languagepath, this.mlng.Path);
			//Language
			settingsLst.Add(ApplicationConstants.languagestr, lngFile);
			//Language ID
			settingsLst.Add(ApplicationConstants.languageIDstr, this.curLngInd);
			//BackgroundColor
			if(cadPictBox.BackColor == Color.Black)
				tmp = ApplicationConstants.blackstr2;
			else tmp = ApplicationConstants.whitestr;
			settingsLst.Add(ApplicationConstants.backcolorstr, tmp);
			//Show entity panel
			settingsLst.Add(ApplicationConstants.showentitystr, this.trvPanel.Visible);
			//Color drawing
			settingsLst.Add(ApplicationConstants.colordrawstr, this.colorDraw);
			//SHXPathCount
			int cn = this.shxFrm.lstDir.Items.Count;
			settingsLst.Add(ApplicationConstants.shxpathcnstr, this.shxFrm.lstDir.Items.Count);
			//SHXPaths
			for(int i = 0; i < cn; i++)
			{
				settingsLst.Add(string.Format("SHXPath_{0}", (i + 1)), this.shxFrm.lstDir.Items[i]);
			}
			#region protect
#if protect
			//License
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				settingsLst.Add("Type_license", "floating");
				settingsLst.Add("Host", this.floatLicFrm.Host);
				settingsLst.Add("Port", this.floatLicFrm.Port);
				settingsLst.Add("Application", this.floatLicFrm.App);
			}
			else
				settingsLst.Add("Type license", "register");
#else
				settingsLst.Add("Type license", "register");
#endif
#endif
			#endregion
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///   ,    
		///    <see cref="CADImport.FaceModule.MultipleLanguage.path">MultipleLanguage.path</see>
		/// </summary>
		#else
		/// <summary>
		/// Loads languages list corresponding to language files from <see cref="CADImport.FaceModule.MultipleLanguage.path">MultipleLanguage.path</see> directory
		/// </summary>
		#endif
		#endregion
		public void ReloadLNG()
		{
			mlng.LoadLngFileList(this.mainMenu.MenuItems[4].MenuItems[0], new System.EventHandler(LanguageSelect_Click));
		}

		#region Help
		#if RUSHELP
		/// <summary>
		///        
		/// </summary>
		/// <param name="val">     (.lng) </param>
		#else
		/// <summary>
		/// Reloads a list of languages available by the specified path
		/// </summary>
		/// <param name="val"></param>
		#endif
		#endregion
		public void ReloadLNG(string val)
		{
			this.mainMenu.MenuItems[4].MenuItems[0].MenuItems.Clear();
			mlng.Path = val;
			if(this.mlng.Language == string.Empty)
				this.curLngInd = 7;
			mlng.LoadLngFileList(this.mainMenu.MenuItems[4].MenuItems[0], new System.EventHandler(LanguageSelect_Click));
			if(this.mainMenu.MenuItems[4].MenuItems[0].MenuItems.Count > this.curLngInd)
				this.mainMenu.MenuItems[4].MenuItems[0].MenuItems[this.curLngInd].Checked = true;
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///      Settings.txt 
		/// </summary>
		#else
		/// <summary>
		/// Loads settings from Settings.txt file
		/// </summary>
		#endif
		#endregion
		public void InitSettings()
		{
			//Load settings
			svSet = new SaveSettings(fileSettingsName);
			settingsLst = svSet.LoadOptions();
			if(settingsLst != null)
			{
				string key = ApplicationConstants.languagepath;
				if(settingsLst.ContainsKey(key))
					this.mlng.Path = Convert.ToString(settingsLst[key]);
			} 
			else
			{
				this.mlng.Path = CADConst.lngPathDefault;
			}
			mlng.LoadLngFileList(this.mainMenu.MenuItems[4].MenuItems[0], new System.EventHandler(LanguageSelect_Click));
			if(settingsLst == null)
				CreateNewSettingsList();
			else
				SetSettings();
		}
	
		#region Help
		#if RUSHELP
		/// <summary>
		///        
		/// </summary>
		#else
		/// <summary>
		/// Sets current language file for all child forms
		/// </summary>
		#endif
		#endregion
		public void SetAllFormsLanguageSettings()
		{
			this.entCreator.OptionsForm.MultipleLanguagesPath = mlng.Path;
			this.entCreator.OptionsForm.LngFileName = lngFile;			
			this.aboutForm.MultipleLanguagesPath = mlng.Path;
			this.aboutForm.LngFileName = lngFile;
			#region FloatingLic
#if floatprotect
			this.floatLicFrm.MultipleLanguagesPath = mlng.Path;
			this.floatLicFrm.LngFileName = lngFile;
#endif
			#endregion
			this.lForm.MultipleLanguagesPath = mlng.Path;
			this.lForm.LngFileName = lngFile;
			this.prtForm.MultipleLanguagesPath = mlng.Path;
			this.prtForm.LngFileName = lngFile;
			this.regFrm.MultipleLanguagesPath = mlng.Path;
			this.regFrm.LngFileName = lngFile;			
			this.rasterSizeForm.MultipleLanguagesPath = mlng.Path;
			this.rasterSizeForm.LngFileName = lngFile;
			this.shxFrm.MultipleLanguagesPath = mlng.Path;
			this.shxFrm.LngFileName = lngFile;
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///    
		/// </summary>
		#else
		/// <summary>
		/// Initializes a class of multiple language support
		/// </summary>
		#endif
		#endregion
		public void InitLng()
		{
			mlng = new MultipleLanguage(this.GetType());
			//Save value of NoName elements
			if(this.mainMenu != null)
				mlng.SaveNameMenuItem(this.mainMenu.MenuItems);
			if(this.ContextMenu != null)
				mlng.SaveNameMenuItem(this.ContextMenu.MenuItems);
			mlng.SaveNameNoNameElement(this.Controls);
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///    
		/// </summary>
		#else
		/// <summary>
		/// Sets a current mode of the controls
		/// </summary>
		#endif
		#endregion
		public void ChangeControlState()
		{
			bool detNullImg = cadImage != null;
			EnableButton(detNullImg);
			bool tmp = false;
			//back color
			if(cadPictBox.BackColor == Color.White)
				tmp = true;
			miWhiteBack.Checked = tmp;
			miBlackBack.Checked = ! tmp;
			if(detNullImg)
			{
				tlbTool.Buttons[3].Pushed = tmp;
				tlbTool.Buttons[4].Pushed = ! tmp;
			}
			else 
			{
				tlbTool.Buttons[3].Pushed = false;
				tlbTool.Buttons[4].Pushed = false;
			}
			//text show
			tmp = this.textVisible;
			miTextsShow.Checked = tmp;
			//dimension visible
			tmp = this.dimVisible;
			miDimShow.Checked = tmp;
			//tree visible
			miEntitiesTree.Checked = this.trvEntity.Visible;
			//color draw
			tmp = this.colorDraw;
			miColorDraw.Checked = tmp;
			miBlackDraw.Checked = ! tmp;
			//use win ellipse
			if(cadImage != null)
				tmp = cadImage.UseWinEllipse;
			else tmp = false;
			miArcsSplit.Checked = ! tmp;
			//show LineWeight 
			miShowLineWeight.Checked = this.showLineWeight;
			//lng
			if(curLngInd < this.mainMenu.MenuItems[4].MenuItems[0].MenuItems.Count)
				this.mainMenu.MenuItems[4].MenuItems[0].MenuItems[curLngInd].Checked = false;
			if(this.mainMenu.MenuItems[4].MenuItems[0].MenuItems.Count > this.curLngInd)
				this.mainMenu.MenuItems[4].MenuItems[0].MenuItems[this.curLngInd].Checked = true;
			tlbTool.Buttons[26].Pushed = this.enableSnap;
			tlbTool.Buttons[7].Pushed = useSHXFonts;
		}

		#region Help
		/// <summary>
		/// Cleans up any resources being used.
		/// </summary>
		/// <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
		///</param>
		#endregion Help
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		private void Open_Click(object sender, System.EventArgs e)
		{
			LoadFile(true);	
			ChangeControlState();
		}

		private void InvalidateImage(object sender, System.EventArgs e)
		{
			this.cadPictBox.Invalidate();
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///      <see cref="CADImport.CADEntity"/>Entity   
		///   <see cref="MainForm.StartPoint"></see>  
		/// <see cref="MainForm.EndPoint"></see> 
		/// </summary>
		#else
		/// <summary>
		/// Sets new position for <see cref="CADImport.CADEntity"/>Entity on offsetting
		/// using <see cref="StartPoint">start</see> and 
		/// <see cref="EndPoint">end</see> positions
		/// </summary>
		#endif
		#endregion
		public void SetNewEntityPosition()
		{
			if((this.startPoint.X != 0)||(this.startPoint.Y != 0))
			{
				DPoint p1 = this.GetRealPoint(this.endPoint.X, this.endPoint.Y);
				DPoint p2 = this.GetRealPoint(this.endPoint.X - this.startPoint.X, this.endPoint.Y - this.startPoint.Y);								
				cadImage.SetNewPosEntitiesExt(p2.X - p1.X, p2.Y - p1.Y, p2.Z - p1.Z, this.cadPictBox.CreateGraphics());
				this.startPoint.X = 0;
				this.startPoint.Y = 0;
				this.endPoint.X = 0;
				this.endPoint.Y = 0;				
			}
		}
		#region Help
#if RUSHELP
		/// <summary>
		///     <see cref="CADImport.CADEntity"/>Entity  
		///   <see cref="CADImport.Professional.Marker"/> 
		///   <see cref="MainForm.StartPoint"></see>  
		/// <see cref="MainForm.EndPoint"></see>  
		/// </summary>
#else
		/// <summary>
		/// Sets new position for <see cref="CADImport.CADEntity"/>Entity 
		/// on editing by <see cref="CADImport.Professional.Marker"/>markers
		/// using <see cref="MainForm.StartPoint">start</see> and 
		/// <see cref="MainForm.EndPoint">end</see> positions
		/// </summary>
#endif
		#endregion
		public void SetNewMarkerPosition()
		{
			if((this.startPoint.X != 0)||(this.startPoint.Y != 0))
			{
				DPoint p1 = this.GetRealPoint(this.endPoint.X, this.endPoint.Y);
				DPoint p2 = this.GetRealPoint(this.endPoint.X - this.startPoint.X, this.endPoint.Y - this.startPoint.Y);
				#region Professional
				#if Professional
				if(this.cadImage.EnableSnap)				
					p2 = cadImage.SnapRealPoint;												
				#endif
				#endregion
				cadImage.SetNewEntityMarkerPos(p2.X - this.cadImage.CurrentMarker.Point.X, p2.Y - this.cadImage.CurrentMarker.Point.Y, p1.Z);
				this.startPoint.X = 0; 
				this.startPoint.Y = 0;
				this.endPoint.X = 0;
				this.endPoint.Y = 0;
			}
		}
		#region Help
		#if RUSHELP
		/// <summary>		
		///    <see cref="MainForm.Image"/>CADImage
		/// </summary>
		#else
		/// <summary>
		/// Clears current <see cref="Image"/>CADImage object
		/// </summary>
		#endif
		#endregion
		public void ClearCADImage()
		{
			lForm.LayerList.Clear();
			lForm.LayerList.SelectItemEvent -= listItemSelectDel;			
			if(cadImage != null)
			{	
				cadImage.Dispose();
				cadImage = null;
			}
			ImageScale = 1.0f;
			ImagePreviousScale = 1.0f;
			Position = new PointF();		
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///     <see cref="MainForm.Image"/>CADImage    
		/// </summary>
		#else
		/// <summary>
		/// Sets current properties for <see cref="MainForm.Image"/>CADImage for loading file
		/// </summary>
		#endif
		#endregion
		public void SetCADImageOptions()
		{
			SetSnapMode();
			cadImage.SelectionMode = SelectionEntityMode.Enabled;
			cadImage.UseDoubleBuffering = false;
			cadImage.UseWinEllipse = false;
			if(cadPictBox.BackColor == Color.White)
				White_Click();
			else 
				Black_Click();
			ViewLayouts();
			if(trvPanel.Visible == true)
				CADImportFace.LoadTreeNodes(trvEntity.Nodes, cadImage);
			if(this.colorDraw == false) 
				DoBlackColor();
			SetLayList();
			DoResize();			
			ObjEntity.cadImage = cadImage;			//for object inspector
			this.prtForm.Image = this.cadImage;
			ObjEntity.propGrid = this.propGrid;		//for object inspector
			this.entCreator.Image = this.cadImage;			
		}

		#region Help
		/// <summary>
		/// Loads a CAD file into the main application's form.
		/// </summary>
		/// <param name="dlg">A value indicating if an <see cref="System.Windows.Forms.OpenFileDialog">OpenFileDialog</see> 
		/// box is invoked for selecting and loading a CAD file. <b>true</b>, if an <see cref="System.Windows.Forms.OpenFileDialog">OpenFileDialog</see> 
		/// box is invoked; otherwise, <b>false</b>.</param>
		#endregion Help
		public void LoadFile(bool dlg)
		{
			SetPictureBoxLoadState(true, string.Empty);			
			if(dlg)
			{
				DialogResult dlgRes = openFileDlg.ShowDialog(this);
				if((dlgRes != DialogResult.OK)||(openFileDlg.FileName == null))
				{
					SetPictureBoxLoadState(false, string.Empty);
					return;
				}			
			}
			ClearCADImage();							
			this.cadImage = CADImage.CreateImageByExtension(openFileDlg.FileName);			
			if(this.cadImage is CADRasterImage)
				(this.cadImage as CADRasterImage).Control = this.cadPictBox;			
			if(this.cadImage == null)
			{
				SetPictureBoxLoadState(false, string.Empty);
				return;
			}					
			CADImage.CodePage = System.Text.Encoding.Default.CodePage;
			CADImage.LastLoadedFilePath = Path.GetDirectoryName(fileName); 
			#region DebugInfo
#if DEBUG
			//DateTime d1, d2;
			//d1 = DateTime.Now;
#endif
			#endregion				
			if(CADConst.IsWebPath(openFileDlg.FileName))
				this.cadImage.LoadFromWeb(openFileDlg.FileName);
			else
				cadImage.LoadFromFile(openFileDlg.FileName);
			#region DebugInfo
#if DEBUG			
			//d2 = DateTime.Now;
			//WriteTimeSpan(string.Format("LoadFromFile - {0}", fileName), @"E:\DebugInfo.txt", d1, d2);
#endif
			#endregion
			SetCADImageOptions();						
			SetPictureBoxPosition(Position);
			EnableButton(true);			
			SetPictureBoxLoadState(false, openFileDlg.FileName);
		}

		private void SetPictureBoxLoadState(bool val, string name)
		{
			if(val)
			{
				this.Cursor = Cursors.WaitCursor;
				this.cadPictBox.Visible = false;
				stBar.Panels[0].Text = ApplicationConstants.loadfilestr;
			}
			else
			{
				this.Cursor = Cursors.Default;
				this.cadPictBox.Visible = true;				
				if(name != String.Empty)
					stBar.Panels[0].Text = name;
			}
		}

		#region Help
		/// <summary>
		/// Creates a list of layouts of the CAD image and sets a current layout. 
		/// </summary>
		#endregion Help
		public void ViewLayouts()
		{			
			if(cadImage == null) 
				return;
			cadImage.SetCurrentLayout(cadImage.DefaultLayoutIndex);
		}

		#region Help
		/// <summary>
		/// Enables or disables buttons of the toolbar and menu items of the main menu.
		/// </summary>
		/// <param name="aVal"><b>true</b> if to enable the buttons and menu items; <b>false</b> if to disable them.</param>
		/// <remarks>The buttons and menu items are enabled after loading a CAD file and become disabled after closing the file.</remarks>
		#endregion Help
		public void EnableButton(bool aVal)
		{
			for(int i = 1; i < 8; i++)
				tlbTool.Buttons[i].Enabled = aVal;
			for(int i = 23; i < 26; i++)
				tlbTool.Buttons[i].Enabled = aVal;
			#region Export
#if Export
			this.miSaveAsDXF.Enabled = aVal;
#endif
			#endregion
			miScale.Enabled = aVal;
			miSaveFile.Enabled = aVal;
			miCopyAsBMP.Enabled = aVal;
			miZoomIn.Enabled = aVal;
			miZoomOut.Enabled = aVal;
			miFit.Enabled = aVal;
			miColorDraw.Enabled = aVal;
			miBlackDraw.Enabled = aVal;
			miWhiteBack.Enabled = aVal;
			miBlackBack.Enabled = aVal;
			miShowLineWeight.Enabled = aVal;
			miArcsSplit.Enabled = aVal;
			miDimShow.Enabled = aVal;
			miTextsShow.Enabled = aVal;
			miLayersShow.Enabled = aVal;			
			miClose.Enabled = aVal;
			miPrint.Enabled = aVal;
			miPrintCustom.Enabled = aVal;
			miPrintPreview.Enabled = aVal;
			miUndo.Enabled = aVal;
			miRedo.Enabled = aVal;
			miDelete.Enabled = aVal;
			miCopy.Enabled = aVal;
			miPaste.Enabled = aVal;
			miCut.Enabled = aVal;
			miSerializeEntities.Enabled = aVal;			
		}

		private void ZoomIn_Click(object sender, System.EventArgs e)
		{
			DoZoomIn();
		}

		#region Help
		/// <summary>
		/// Increases a scale of the CAD image in two times.
		/// </summary>
		#endregion Help
		public void DoZoomIn()
		{
			if(cadImage == null) 
				return;
			PreviousPosition = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			ImageScale = ImageScale * 2.0f;
			cadPictBox.Invalidate();			
			SetPictureBoxPosition(Position);			
			this.stopSnap = false;	
		}

		private void ZoomOut_Click(object sender, System.EventArgs e)
		{
			DoZoomOut();
		}

		#region Help
		/// <summary>
		/// Decreases a scale of the CAD image in two times.
		/// </summary>
		#endregion Help
		public void DoZoomOut()
		{
			if(cadImage == null) return;
			PreviousPosition = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			ImageScale = ImageScale / 2.0f;
			cadPictBox.Invalidate();			
			SetPictureBoxPosition(Position);			
			this.stopSnap = false;
		}

		private void cadPictBox_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			DrawCADImage(e.Graphics);
			stBar.Panels[1].Text = RealScale;							
		}

		#region Help
		/// <summary>
		/// Draws a CAD image from the loaded CAD file.
		/// </summary>
		/// <param name="gr">A <see cref="System.Drawing.Graphics">Graphics</see> object used to draw the CAD entities.</param>
		#endregion Help
		public void DrawCADImage(Graphics gr)
		{
			if(cadImage == null) return;							
			Shift();
			RectangleF tmp = ImageRectangleF;								
			SetSizePictureBox(new Size((int)tmp.Width, (int)tmp.Height));				
			try
			{
				cadImage.Draw(gr, tmp);								
			}
			catch
			{
				return;				
			}
		}
		#region Help
		#if RUSHELP	
		/// <summary>
		///    <see cref="MainForm.EditorCADPictureBox">CADPictureBox</see>,
		///     
		/// </summary>
		/// <param name="sz"> <see cref="System.Drawing.Size"></see></param>
		#else
		/// <summary>
		/// Sets new <see cref="MainForm.EditorCADPictureBox">CADPictureBox</see> dimensions,
		/// where an image is drawn
		/// </summary>
		/// <param name="sz"><see cref="System.Drawing.Size">Size</see> to set</param>
		#endif
		#endregion
		public void SetSizePictureBox(Size sz)
		{
			this.cadPictBox.SetVirtualSizeNoInvalidate(sz);			
		}

		private void SetPictureBoxPosition(PointF value)
		{
			int w1, h1;
			if(value.X > 0)
				w1 = 0;
			else
				w1 = (int)Math.Abs(value.X);
			if(w1 > this.cadPictBox.VirtualSize.Width)
				w1 = this.cadPictBox.VirtualSize.Width;
			if(value.Y > 0)
				h1 = 0;
			else
				h1 = (int)Math.Abs(value.Y);
			if(h1 > this.cadPictBox.VirtualSize.Height)
				h1 = this.cadPictBox.VirtualSize.Height;
			this.cadPictBox.SetPositionNoInvalidate(new Point(w1, h1));
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///    
		/// (      <b>true</b>)
		/// </summary>
		/// <param name="x">   X</param>
		/// <param name="y">   X</param>
		/// <returns>      <b>true</b></returns>
		#else	
		/// <summary>
		/// Verifies suitability of the entity moving
		/// </summary>
		/// <param name="x">X axis position</param>
		/// <param name="y">Y  axis position</param>
		/// <returns>Returns <b>true</b> if the offset is minimal</returns>
		#endif
		#endregion
		public bool ChancelMove(int x, int y)
		{
			if(((Math.Abs(cX - x) < 2)&&(Math.Abs(cY - y) < 2))||
				(this.cadImage.SelectEntitiesCount == 0))					
			{				
				return true;
			}
			return false;
		}
		
		private bool ChangeEntityOnMouseUp(Point pt)
		{
			#region Professional
#if Professional					
			if(cadImage.CurrentMarker != null)
			{
				if(! ChancelMove(pt.X, pt.Y))
				{
					SetNewMarkerPosition();				
					return true;
				}
			}
			else
			{				
				if(! CreateNewEntity)
				{
					if(! ChancelMove(pt.X, pt.Y))
					{
						SetNewEntityPosition();				
						return true;
					}
				}
			}			
			return false;
#else
			return false;
#endif

			#endregion
		}

		private void cadPictBox_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{						
			if(cadImage == null) return;					
			if(e.Button != MouseButtons.Right)						
				ChangeEntityOnMouseUp(new Point(e.X, e.Y));					
			detMouseDown = false;			
			if(this.clipRectangle.Enabled)
				if((this.clipRectangle.Type == RectangleType.Zooming))
					UseClipRect();									
			if(! CreateNewEntity)
			{				
				this.stopSnap = true;
				cadPictBox.Invalidate();									
			}
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///   <see cref="CADImport.FaceModule"/>ClipRect
		///   <see cref="CADImport.CADEntityCollection"> </see>
		/// </summary>
		#else
		/// <summary>
		/// Enables <see cref="CADImport.FaceModule"/>ClipRect tool
		/// for <see cref="CADImport.CADEntityCollection">set of entities</see> selection
		/// </summary>
		#endif
		#endregion
		public void UseClipRect()
		{
			if((this.clipRectangle.ClientRectangle.Width <= 10)||
				(this.clipRectangle.ClientRectangle.Height <= 10)) 
				return; 					
			if(MultipleSelect()) 
				return;
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///     ,   
		///     
		/// </summary>
		/// <param name="x">   X</param>
		/// <param name="y">   Y</param>
		/// <returns><b>true</b>           </returns>	
		#else
		/// <summary>
		/// Selects an entity by the specified coordinates.
		/// Sets a number of settings on entity selection.
		/// </summary>
		/// <param name="x">X axis position</param>
		/// <param name="y">Y axis position</param>
		/// <returns>Returns <b>true</b> if there is an entity in specified position and it is selected</returns>
		#endif
		#endregion
		public bool SelectEntity(int x, int y)
		{		
			if(this.CreateNewEntity) 
				return false;
			moveMarker = false;
			if(cadImage == null) 
				return false;
			bool det = this.cadImage.SelectEntitiesCount == 0;
			CADEntity ent = cadImage.SelectExt(x, y, ! cntrlDown, true);
			if(ent != null) 
			{				
				moveMarker = true;		
				this.stopSnap = true;				
			}			
			else
			{
				if(! det)
				{	
					this.stopSnap = true;					
				}
			}			
			return false;
		}
		
		private void cadPictBox_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{			
			if(e.Button != MouseButtons.Right)			
			{
				if(! CreateNewEntity)
					DisableEntityCreator();				
				if(this.cadImage != null)
					if(cadImage.SelectionMode == SelectionEntityMode.Enabled)
					{							
						if(SelectEntity(e.X, e.Y))
						{																
							return;										
						}						
						if((this.cadImage.SelectedEntities != null)&&(! CreateNewEntity))
							this.cadImage.InvalidateSelectEntity(this.cadPictBox, this.ImageScale);							
					}	
			}	
			cX = e.X;
			cY = e.Y;			
			detMouseDown = true;		
			if(CreateNewEntity)
				if(EnableNewEntityCreator()) 				
				{		
					return;
				}
			if((e.Button == MouseButtons.Left)&&(! moveMarker))
				this.clipRectangle.EnableRect(RectangleType.Zooming, new Rectangle(e.X, e.Y, 0, 0));			
		}

		private void DisableEntityCreator()
		{
			if(this.entCreator.Enabled)
				if((this.entCreator.Type == CreatorType.Pen)||((this.entCreator.Type == CreatorType.Polyline) 
					&& this.entCreator.EndPoly)||(this.entCreator.Type == CreatorType.Point))
					this.entCreator.Disable();			
		}

		private bool EnableNewEntityCreator()
		{		
			CreatorType seltp = CreatorType.Undetected;
			switch(this.addEntity)
			{
				case 0:
					seltp = CreatorType.Image;					
					break;
				case 1:
					seltp = CreatorType.Solid;										
					break;
				case 2:
					seltp = CreatorType.Line;																				
					break;					
				case 3:
					seltp = CreatorType.Rectangle;										
					break;					
				case 4:
					seltp = CreatorType.Ellipse;
					break;					
				case 5:
					seltp = CreatorType.Text;					
					break;										
				case 6:
					seltp = CreatorType.Polyline;
					break;					
				case 7:
					seltp = CreatorType.Pen;
					break;					
				case 8:
					seltp = CreatorType.Hatch;
					break;	
				case 9:
					seltp = CreatorType.Cloud;
					break;	
				case 10:
					seltp = CreatorType.MText;
					break;	
				case 11:
					seltp = CreatorType.Point;
					break;	
			}					
			if(seltp != CreatorType.Undetected)	
			{
				this.entCreator.EnableCreator(seltp);											
				curAddEntityType = seltp;							
				return true;
			}
			return false;
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///   <see cref="CADImport.CADEntity"/>
		/// </summary>
		/// <param name="val">   <see cref="CADImport.CADEntity"/></param>
		#else
		/// <summary>
		/// Finishes adding of the <see cref="CADImport.CADEntity"/>entity
		/// </summary>
		/// <param name="val">A finishing flag of the <see cref="CADImport.CADEntity"/>entity adding</param>
		#endif
		#endregion
		public void EndAddEntity(bool val)
		{
			if(curAddEntityType != CreatorType.Undetected)
			{
				SetAddEntityMode(curAddEntityType);
				SetEnableChecked();				
				ChangeControlState();
			}
			this.detMouseDown = val;
		}	
		#region Help
		#if RUSHELP
		/// <summary>
		///   <see cref="CADImport.CADEntity"/>    
		/// <see cref="CADImport.Professional.Marker"/> 
		///   <see cref="CADImport.CADEntity"/> 
		/// </summary>
		/// <param name="pt"></param>
		/// <returns></returns>
		#else
		/// <summary>
		/// Changes <see cref="CADImport.CADEntity"/>entity position by
		/// changing <see cref="CADImport.CADEntity"/>marker position or
		/// <see cref="CADImport.CADEntity"/>entity dragging 
		/// </summary>
		/// <param name="pt">New coordinates</param>
		/// <returns>Returns <b>true</b> if the changing is successful</returns>
		#endif
		#endregion
		public bool ChangeEntityOnMouseMove(Point pt)
		{
			#region Professional
#if Professional
			if(this.cadImage.SelectEntitiesCount != 0)
			{		
				if(this.CreateNewEntity) return true;
				if(cadImage.CurrentMarker != null)
				{									
					ChangeMarkerPosition(pt.X, pt.Y);
					return true;
				}
				else
				{						
					ChangeEntityPositionExt(pt.X, pt.Y);
					return true;					
				}				
			}			
#endif		
			#endregion
			return false;
		}

		private void cadPictBox_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{			
			if(cadImage == null) 
				return;	
			if(!this.cadPictBox.Focused)
			{				
				this.cadPictBox.Focus();			
				this.cadPictBox.Invalidate();
			}
			if((! stopSnap)&&(this.cadPictBox.ClientRectangle.Contains(e.X, e.Y)))			
			{			
				RectangleF tmpRect = ImageRectangleF;
				cadImage.DrawSnapTrace(e.X, e.Y, this.cadPictBox, tmpRect);								
			}
			else
			{
				this.cadImage.RefreshSnapTrace(this.cadPictBox);
				this.stopSnap = false;
			}
			if(detMouseDown)
			{
				if(e.Button != MouseButtons.Right)
					if(ChangeEntityOnMouseMove(new Point(e.X, e.Y)))										
						return;					
				if((!this.clipRectangle.Enabled) && (!this.CreateNewEntity))
				{					
					LeftImagePosition -= (cX - e.X);
					TopImagePosition -= (cY - e.Y);					
					cadPictBox.Invalidate();
					SetPictureBoxPosition(Position);					
				}
				cX = e.X;
				cY = e.Y;				
			}			
			PreviousPosition = new PointF(e.X, e.Y);
			DPoint vPt = GetRealPoint(e.X, e.Y);
			stBar.Panels[2].Text = string.Format("{0} : {1} : 0", vPt.X, vPt.Y);				
		}	

		private void ChangeEntityPositionExt(int x, int y)
		{
			Graphics gr = this.cadPictBox.CreateGraphics();		
			this.cadImage.DrawMoveEntityTrace(-this.startPoint.X, -this.startPoint.Y, this.cadPictBox);
			this.cadImage.DrawMoveEntityTrace(-(cX - x), -(cY - y), this.cadPictBox);
			this.startPoint.X = (cX - x);
			this.startPoint.Y = (cY - y);			
		}
		#region Professional
#if Professional
		private void ChangeMarkerPosition(int x, int y)
		{
			Graphics gr = this.cadPictBox.CreateGraphics();
			//repaint old
			if((this.startPoint.X != 0)||(this.startPoint.Y != 0))
			{
				this.cadImage.DrawChangeEntity(this.startPoint.X, this.startPoint.Y, this.cadPictBox);  				
			}
			else
			{
				endPoint.X = x;
				endPoint.Y = y;
			}
			//paint new
			this.cadImage.DrawChangeEntity((cX - x), (cY - y), this.cadPictBox);  
			this.startPoint.X = (cX - x);
			this.startPoint.Y = (cY - y);
		}
#endif
		#endregion

		private bool MultipleSelect()
		{
			if(cadImage.SelectionMode == SelectionEntityMode.Enabled)
			{
				int l = this.clipRectangle.ClientRectangle.Left;
				int t = this.clipRectangle.ClientRectangle.Top;				
				DPoint pt1 = this.GetRealPoint(l, t);
				DPoint pt2 = this.GetRealPoint(this.clipRectangle.ClientRectangle.Right, this.clipRectangle.ClientRectangle.Bottom);
				DRect tmpRect = new DRect(pt1.X, pt1.Y, pt2.X, pt2.Y);				
				Region rg = cadImage.GetSelectEntityRegion(new Rectangle((int)l, (int)t, this.clipRectangle.ClientRectangle.Width, this.clipRectangle.ClientRectangle.Height));							
				#region Professional
#if Professional
				this.propGrid.SelectedObject = new CADImport.ObjEntities(cadImage.MultipleSelectExt(tmpRect, true, true));				
#else
				cadImage.MultipleSelect(tmpRect, true);
#endif
				#endregion				
				cadPictBox.Invalidate(rg);
				rg.Dispose();
				return true;
			}
			return false;
		}

		#region Help
		/// <summary>
		/// Gets a three-dimensional point of the current CAD image from the specified screen point.
		/// </summary>
		/// <param name="x">An X coordinate of the screen point.</param>
		/// <param name="y">A Y coordinate of the screen point.</param>
		/// <returns>A three-dimensional point.</returns>
		#endregion Help
		public DPoint GetRealPoint(int x, int y)
		{			
			RectangleF tmpRect = ImageRectangleF;	
			return CADConst.GetRealPoint(x, y, this.cadImage, tmpRect);
		}		

		private void MoveToPosition(ref DPoint point, DPoint aPos, int direction)
		{  
			point.X = point.X + direction * aPos.X;
			point.Y = point.Y + direction * aPos.Y;
			point.Z = point.Z + direction * aPos.Z;
		}

		private void miZoom_Click(object sender, System.EventArgs e)
		{
			if(cadImage == null) 
				return;
			float i = ImageScale;
			switch((sender as MenuItem).Index)
			{
				case 0:
					i = 0.1f;
					break;
				case 1:
					i = 0.25f;
					break;
				case 2:
					i = 0.5f;
					break;
				case 3:
					i = 1.0f;
					break;
				case 4:
					i = 2.0f;
					break;
				case 5:
					i = 4.0f;
					break;
				case 6:
					i = 8.0f;
					break;
			}
			ResetScaling();
			this.PreviousPosition = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			ImageScale = i;
			cadPictBox.Invalidate();
			SetPictureBoxPosition(Position);			
		}

		#region Help
		/// <summary>
		/// Paints the CAD image background in the white color.
		/// </summary>
		#endregion Help
		public void White_Click()
		{
			cadPictBox.BackColor = Color.White;
			if(this.cadImage != null)
				this.cadImage.BackgroundColor = Color.White;
			this.trvEntity.BackColor = Color.White;
			this.trvEntity.ForeColor = Color.Black;
			if(cadImage != null)
				cadImage.DefaultColor = Color.Black;
			if(this.clipRectangle != null)
				this.clipRectangle.Color = Color.Black;
		}

		#region Help
		/// <summary>
		/// Paints the CAD image background in the black color.
		/// </summary>
		#endregion Help
		public void Black_Click()
		{
			cadPictBox.BackColor = Color.Black;
			if(this.cadImage != null)
				this.cadImage.BackgroundColor = Color.Black;
			this.trvEntity.BackColor = Color.Black;
			this.trvEntity.ForeColor = Color.White;			
			if(cadImage != null)
				cadImage.DefaultColor = Color.White;
			if(this.clipRectangle != null)
				this.clipRectangle.Color = Color.White;
		}

		private void tlbTool_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{		
			switch(tlbTool.Buttons.IndexOf(e.Button))
			{
				case 0:
					LoadFile(true);
					break; 
				case 1:
					DoZoomIn();
					break; 
				case 2:
					DoZoomOut();
					break;
				case 3:
					White_Click();
					break;
				case 4:
					Black_Click();
					break;
				case 5:
					SetLayList();
					lForm.ShowDialog();
					break;
				case 6:
					SaveAsImage();
					break;
				case 7:
					if(MessageBox.Show(ApplicationConstants.messagestr, ApplicationConstants.appnamestr, MessageBoxButtons.YesNo) == DialogResult.Yes)
						ChangeTextsType(! useSHXFonts);
					break;
				case 8:
					SetAddEntityMode(CreatorType.Image);
					break;
				case 9:
					SetAddEntityMode(CreatorType.Solid);
					break;
				case 10:
					SetAddEntityMode(CreatorType.Line);
					break;
				case 11:
					SetAddEntityMode(CreatorType.Rectangle);
					break;
				case 12:
					SetAddEntityMode(CreatorType.Ellipse);
					break;
				case 13:
					SetAddEntityMode(CreatorType.Text);
					break;
				case 14:
					SetAddEntityMode(CreatorType.Polyline);					
					break;
				case 15:
					SetAddEntityMode(CreatorType.Pen);
					break;
				case 16:
					SetAddEntityMode(CreatorType.Hatch);					
					break;
				case 17:
					SetAddEntityMode(CreatorType.Cloud);
					break;
				case 18:
					SetAddEntityMode(CreatorType.MText);
					break;
				case 19:
					SetAddEntityMode(CreatorType.Point);
					break;
				case 20:
					this.entCreator.OptionsForm.ShowDialog();
					break;
				case 21:
					CreateNewImage();
					break;
				case 22:
					GetExtents();
					break;
				case 23:
					RemoveEntity();
					break;
				case 24:
					UndoChangeEntity();
					break;
				case 25:
					RedoChangeEntity();
					break;
				case 26:
					this.enableSnap = ! this.enableSnap;
					SetSnapMode();
					break;
			}
			SetEnableChecked();			
			ChangeControlState();					
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///   () 
		/// </summary>
		#else
		/// <summary>
		/// Sets snap mode
		/// </summary>
		#endif
		#endregion
		public void SetSnapMode()
		{			
			if(this.cadImage != null)
				this.cadImage.EnableSnap = this.enableSnap;
		}

		private void UndoChangeEntity()
		{
			this.cadImage.Undo();
			CADImportFace.LoadTreeNodes(trvEntity.Nodes, cadImage);
			this.cadPictBox.Invalidate();			
		}

		private void RedoChangeEntity()
		{
			this.cadImage.Redo();
			this.cadPictBox.Invalidate();
		}

		private void RemoveEntity()
		{
			if(this.cadImage != null)
				this.cadImage.RemoveEntity();			
			this.cadPictBox.Invalidate();
		}

		private void SetAddEntityMode(CreatorType val)
		{
			int tmpVal = addEntity;
			addEntity = -1;
			switch(val)
			{
				case CreatorType.Image:
					if(tmpVal != 0)
						addEntity = 0;											
					break;
				case CreatorType.Solid:
					if(tmpVal != 1)
						addEntity = 1;											
					break;
				case CreatorType.Line:
					if(tmpVal != 2)
						addEntity = 2;											
					break;					
				case CreatorType.Rectangle:
					if(tmpVal != 3)
						addEntity = 3;											
					break;					
				case CreatorType.Ellipse:
					if(tmpVal != 4)
						addEntity = 4;											
					break;					
				case CreatorType.Text:
					if(tmpVal != 5)
						addEntity = 5;											
					break;										
				case CreatorType.Polyline:
					if(tmpVal != 6)
						addEntity = 6;											
					break;					
				case CreatorType.Pen:
					if(tmpVal != 7)
						addEntity = 7;											
					break;					
				case  CreatorType.Hatch:
					if(tmpVal != 8)
						addEntity = 8;											
					break;	
				case CreatorType.Cloud:
					if(tmpVal != 9)
						addEntity = 9;											
					break;	
				case CreatorType.MText:
					if(tmpVal != 10)
						addEntity = 10;											
					break;	
				case CreatorType.Point:
					if(tmpVal != 11)
						addEntity = 11;											
					break;	
			}						
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///  <b>true</b>       
		///    
		/// </summary>
		#else
		/// <summary>
		/// Gets <b>true</b> if entity creating mode is enabled at the moment
		/// </summary>
		#endif
		#endregion
		public bool CreateNewEntity
		{
			get
			{
				if((this.cadImage == null)&&(addEntity == 8)) 
					return false; 
				if(this.cadImage != null)
					if((this.cadImage.SelectedEntities.Count == 0)&&(addEntity == 8)) 
						return false;
				return (addEntity != -1);
			}
		}

		private void GetExtents()
		{
			if(cadImage == null) return;
			cadImage.GetExtents();
			DoResize();
		}
		#region Help
		/// <summary>
		/// Create new <see cref="CADImport.CADImage">CADImage</see> object
		/// </summary>
		#endregion
		public void CreateNewImage()
		{
			if(cadImage != null)
				cadImage.Dispose();
			ImageScale = 1.0f;
			ImagePreviousScale = 1.0f;
			Position = new PointF();
			cadImage = new CADImage();
			cadImage.SelectionMode = SelectionEntityMode.Enabled;
			this.cadImage.UseDoubleBuffering = false;
			if(cadPictBox.BackColor == Color.White)
				White_Click();
			else Black_Click();
			this.cadImage.InitialNewImage();
			EnableButton(true);
			this.trvEntity.Nodes.Clear();				
			DoResize();			
			ObjEntity.cadImage = cadImage;
			ObjEntity.propGrid = this.propGrid;
			cadImage.UseWinEllipse = false;
			this.entCreator.Image = this.cadImage;
			prtForm.Image = this.cadImage;
			SetLayList();			
		}

		private void SetEnableChecked()
		{			
			tlbTool.Buttons[8].Pushed = (addEntity == 0);
			tlbTool.Buttons[9].Pushed = (addEntity == 1);
			tlbTool.Buttons[10].Pushed = (addEntity == 2);
			tlbTool.Buttons[11].Pushed = (addEntity == 3);
			tlbTool.Buttons[12].Pushed = (addEntity == 4);
			tlbTool.Buttons[13].Pushed = (addEntity == 5);
			tlbTool.Buttons[14].Pushed = (addEntity == 6);
			tlbTool.Buttons[15].Pushed = (addEntity == 7);
			tlbTool.Buttons[16].Pushed = (addEntity == 8);
			tlbTool.Buttons[17].Pushed = (addEntity == 9);
			tlbTool.Buttons[18].Pushed = (addEntity == 10);
			tlbTool.Buttons[19].Pushed = (addEntity == 11);			
		}
		
		private void ChangeTextsType(bool val)
		{
			useSHXFonts = val;
			CADConst.UseSHXFonts = useSHXFonts;
			CADConst.UseTTFFonts = ! useSHXFonts;
			CADConst.UseMultyTTFFonts = ! useSHXFonts;			
			this.ReOpen();
		}
	
		#region Help
		/// <summary>
		/// Reopens a file after selecting or deselecting SHX fonts.
		/// </summary>
		/// <remarks>This method is also called when the paths to the 
		/// files containing SHX fonts have been edited in the <b>SHX Paths</b> window 
		/// invoked by the <b>SHX Fonts</b> menu item.</remarks>
		#endregion Help
		public void ReOpen()
		{
			if(openFileDlg.FileName != null)
				if(File.Exists(openFileDlg.FileName))
					this.LoadFile(false);
		}
		#region Help
		/// <summary>
		/// Sets the scale and position of the CAD image to its original values.
		/// </summary>
		#endregion Help
		public void ResetScaling()
		{
			ImageScale = 1.0f;
			ImagePreviousScale = 1.0f;
			LeftImagePosition = (cadPictBox.ClientRectangle.Width - VisibleAreaSize.Width) / 2.0f;
			TopImagePosition = (cadPictBox.ClientRectangle.Height - VisibleAreaSize.Height) / 2.0f;
			cadPictBox.Invalidate();
		}

		private void AddRecordToLayerList(string name, Color color, bool visible, bool freeze)
		{
			ListItem lstItem = new ListItem();			
			lstItem.Text = name;								
			lForm.LayerList.Items.Add(lstItem);
			lstItem = new ListItem();
			if(visible)				
				lstItem.ImageIndex = 7;							
			else
				lstItem.ImageIndex = 8;							
			lForm.LayerList.Items.Add(lstItem);
			lstItem = new ListItem();
			if(freeze)
				lstItem.ImageIndex = 3;				
			else
				lstItem.ImageIndex = 4;				
			lForm.LayerList.Items.Add(lstItem);	
			lstItem = new ListItem();
			lstItem.DrawColorRect = true;
			lstItem.BackColor = color;			
			lForm.LayerList.Items.Add(lstItem);			
		}
		#region Help
#if RUSHELP
		/// <summary>
		///   <see cref="CADImport.CADLayer.Visible">CADLayer.Visible</see>      <see cref="CADImport.CADLayer"></see>
		/// </summary>
		/// <param name="visible"> </param>
		/// <param name="index">    <see cref="CADImport.CADConverter.Layers"></see></param>
#else
		/// <summary>
		/// Sets <see cref="CADImport.CADLayer.Visible">CADLayer.Visible</see> parameter for the layer
		/// by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="visible">Setting value</param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers"></see> collection</param>
#endif
		#endregion
		public void SetVisibleLayer(bool visible, int index)
		{
			cadImage.SetLayerVisible(index, visible, lForm.detInvalidate);			
			this.cadImage.ClearSelection();
			this.cadImage.ClearMarkers();
			InitialNewPrintPages();			
			cadPictBox.Invalidate();			
		}
		#region Help
#if RUSHELP
		/// <summary>
		///   <see cref="CADImport.CADLayer.Frozen">CADLayer.Frozen</see>      <see cref="CADImport.CADLayer"></see>
		/// </summary>
		/// <param name="visible"> </param>
		/// <param name="index">    <see cref="CADImport.CADConverter.Layers"></see></param>
#else
		/// <summary>
		/// Sets <see cref="CADImport.CADLayer.Frozen">CADLayer.Frozen</see> parameter for the layer
		/// by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="visible">Setting value</param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers">layers</see> collection</param>
#endif
		#endregion
		public void SetFreezeLayer(bool visible, int index)
		{
			cadImage.SetLayerFreeze(index, visible, lForm.detInvalidate);			
			InitialNewPrintPages();
			cadPictBox.Invalidate();			
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///        <see cref="CADImport.CADLayer"></see>
		/// </summary>
		/// <param name="color"> <see cref="System.Drawing.Color"></see></param>
		/// <param name="index">    <see cref="CADImport.CADConverter.Layers"></see></param>
		#else
		/// <summary>
		/// Sets layer color by the specified <see cref="CADImport.CADLayer">layer</see> index
		/// </summary>
		/// <param name="color">Setting <see cref="System.Drawing.Color">color</see></param>
		/// <param name="index">An index of the layer in the <see cref="CADImport.CADConverter.Layers"></see> collection</param>

		#endif
		#endregion
		public void SetColorLayer(Color color, int index)
		{
			cadImage.SetLayerColor(index, color, lForm.detInvalidate);				
			InitialNewPrintPages();
			cadPictBox.Invalidate();
		}

		private void InitialNewPrintPages()
		{
			this.prtForm.Image = this.cadImage;
			this.prtForm.CreateNewPages();	
		}

		private void SetLayList()
		{
			if((lForm.LayerList.Items.Count != 0)||(cadImage == null)) 
				return;
			ObjEntity.LayersList = new string[cadImage.LayersCount];
			string tmp;
			Color colorLay;					
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr1, ApplicationConstants.namestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr2, ApplicationConstants.visiblestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr4, ApplicationConstants.freezestr, 0));
			lForm.LayerList.Columns.Add(new ListHeader(ApplicationConstants.headstr3, ApplicationConstants.colorstr, 0));			
			for(int I = 0; I < cadImage.LayersCount; I++)
			{
				tmp = cadImage.GetLayerName(I);
				colorLay = cadImage.GetLayerColor(I);
				ObjEntity.LayersList[I] = tmp;
				AddRecordToLayerList(tmp, colorLay, cadImage.GetLayerVisible(I), cadImage.GetLayerFreeze(I));				
			}		
			lForm.LayerList.SetAutoSize();
			lForm.LayerList.SelectItemEvent += listItemSelectDel;			
		}

		private void ListSelect(ArrayList item, ListEventSelectArgs e)
		{
			ListItem itm1 =((ListItem)lForm.LayerList.Items[e.ItemIndex]);
			int index = e.ItemIndex / lForm.LayerList.Columns.Count;
			CADImportFace.SelectLayer(index, this.cadImage);
			switch(itm1.ImageIndex)
			{
				case 7:
					itm1.ImageIndex = 8;	
					SetVisibleLayer(false, index);				
					break;
				case 8:
					itm1.ImageIndex = 7;										
					SetVisibleLayer(true, index);
					break;
				case 4:
					itm1.ImageIndex = 3;
					SetFreezeLayer(true, index);
					break;
				case 3:
					itm1.ImageIndex = 4;
					SetFreezeLayer(false, index);
					break;
			}
			if(itm1.DrawColorRect)
			{
				this.setColorDlg.Color = itm1.BackColor;
				if(this.setColorDlg.ShowDialog() == DialogResult.OK)
				{
					SetColorLayer(this.setColorDlg.Color, index);	
					itm1.BackColor = this.setColorDlg.Color;
					this.lForm.LayerList.InvalidateCell(e.ItemIndex);			
				}
			}
			lForm.LayerList.InvalidateCell(e.ItemIndex);
		}

		#region Help
		/// <summary>
		/// Resizes the CAD image to fit drawing's bounds.
		/// </summary>
		/// <remarks>This method is invoked when changing layout or working with 3D Orbit tool.</remarks>
		#endregion Help
		public void DoResize()
		{				
			if(cadImage == null) 
				return;
			float wh = (float)(cadImage.AbsWidth / cadImage.AbsHeight); 
			float new_wh = (float)cadPictBox.ClientRectangle.Width / (float)cadPictBox.ClientRectangle.Height;
			if(cadImage is CADRasterImage)
				visibleArea = new SizeF((float)cadImage.AbsWidth, (float)cadImage.AbsHeight);
			else 
				visibleArea = cadPictBox.Size;
			if(new_wh > wh)
				visibleArea.Width = visibleArea.Height * wh;
			else 
			{
				if(new_wh < wh)
					visibleArea.Height = visibleArea.Width / wh;
				else
					visibleArea = cadPictBox.Size;
			}
			LeftImagePosition = (cadPictBox.ClientRectangle.Width - visibleArea.Width) / 2.0f;
			TopImagePosition = (cadPictBox.ClientRectangle.Height - visibleArea.Height) / 2.0f;				
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Closes a currently open CAD file.
		/// </summary>
		#endregion Help
		public void CloseFile()
		{
			cadImage.Dispose();
			cadImage = null;
			this.entCreator.Image = null;			
			EnableButton(false);
			this.propGrid.SelectedObject = null;
			this.trvEntity.Nodes.Clear();
			this.cadPictBox.Invalidate();
		}
		
		#region Help
		/// <summary>
		/// Makes the layers of the loaded CAD image visible or invisible.
		/// </summary>
		/// <param name="sender">A <see cref="System.Windows.Forms.CheckedListBox">CheckedListBox</see> that initializes changing a visibility of the layers.</param>
		/// <param name="e">An <see cref="System.Windows.Forms.ItemCheckEventArgs">ItemCheckEventArgs</see> object that provides data for the event of changing a layer visibility.</param>
		#endregion Help
		public void chLay_ItemCheck(object sender, System.Windows.Forms.ItemCheckEventArgs e)
		{
			if(e.NewValue == CheckState.Checked)
				cadImage.SetLayerVisible(e.Index, true, lForm.detInvalidate);
			else if(e.NewValue == CheckState.Unchecked) 
				cadImage.SetLayerVisible(e.Index, false, lForm.detInvalidate);
			cadPictBox.Invalidate();
		}

		private void MainForm_Deactivate(object sender, System.EventArgs e)
		{
			detMouseDown = false;
		}

		private void MainForm_Resize(object sender, System.EventArgs e)
		{
			DoResize();
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			CreateNewSettingsList();
			svSet.SaveOptions(settingsLst);
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.DisableFloating();
			}
			else
			{
				Protection.CloseApplication();
			}
#else
				Protection.CloseApplication();
#endif
#endif
			#endregion
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{			
			this.shxFrm.AddDefaultSHXPaths();
			VisibleAreaSize = cadPictBox.Size;			
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.FloatingLicRegister();
			}
			else	
			{
				Protection.Register();
			}
#else
			Protection.Register();
#endif
#endif
			#endregion
			if(fileName != null)
				if(File.Exists(fileName))
				{
					this.openFileDlg.FileName = fileName;
					LoadFile(false);
				}
			#region demo
#if Demo
				DemoForm d1 = new DemoForm();
				d1.ShowDialog();
#endif
			#endregion demo
			ChangeControlState();
		}

		private void miAbout_Click(object sender, System.EventArgs e)
		{
			aboutForm.ShowDialog();
		}

		private void miOpenFile_Click(object sender, System.EventArgs e)
		{
			LoadFile(true);
		}

		private void miExit_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void miSaveAsDXF_Click(object sender, System.EventArgs e)
		{
			if(this.cadImage == null) 
				return;
			if(this.saveDXFDlg.ShowDialog() != DialogResult.OK) 
				return;
			SaveAsDXF(this.saveDXFDlg.FileName);
		}

		private void SaveAsDXF(string fName)
		{
			#region Export
#if Export
			if(cadImage == null) return;
			if(cadImage is CADRasterImage) 
			{
				return;
			}
			CADImport.Export.DirectCADtoDXF.CADtoDXF vExp = new CADImport.Export.DirectCADtoDXF.CADtoDXF(cadImage);
			vExp.SaveToFile(fName);
#else
				MessageBox.Show("Not supported in current version!", "CADImport .Net");
#endif
			#endregion Export			
		}

		private void miSHXFontsPaths_Click(object sender, System.EventArgs e)
		{
			shxFrm.ShowDialog();
		}

		private void miRegister_Click(object sender, System.EventArgs e)
		{
			#region protect
#if protect
			regFrm.ShowDialog();
#endif
			#endregion protect
		}

		private void miFloatLic_Click(object sender, System.EventArgs e)
		{
			#region protect
#if floatprotect
			floatLicFrm.ShowDialog();
#endif
			#endregion
		}

		private void propGrid_PropertyValueChanged(object s, System.Windows.Forms.PropertyValueChangedEventArgs e)
		{
			this.cadImage.ClearMarkers();
			this.cadImage.DoSelectEntities(this.cadImage.SelectedEntities);			
			this.cadPictBox.Invalidate();
		}

		private void trvEntity_AfterCheck(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			CADImportFace.DoCheckTreeNode(trvEntity.Nodes, e.Node, this.cadImage, this.cadPictBox);
			cadPictBox.Invalidate();
		}

		private void trvEntity_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			CADImportFace.DecodeEntity(e.Node, cadImage, this.cadPictBox, this.ImageScale);
		}

		private void miPrint_Click(object sender, System.EventArgs e)
		{
			cadImage.Print(true, true);
		}

		private void miClose_Click(object sender, System.EventArgs e)
		{
			CloseFile();
			this.ChangeControlState();
		}

		private void miPrintPreview_Click(object sender, System.EventArgs e)
		{
			this.DoExtentsForPrint();
			this.prtForm.TypePage = CADImport.Printing.DrawingSize.Fit;						
			printPrevDlg.Document = prtForm.Print(false);	
			printPrevDlg.ShowDialog();
			if(printPrevDlg.Document != null)
				printPrevDlg.Document.Dispose();
		}

		#region Help
		/// <summary>
		/// Invokes a <see cref="System.Windows.Forms.SaveFileDialog">SaveFileDialog</see> box where 
		/// a user can select a directory and format for saving the current CAD image.
		/// </summary>
		/// <remarks><p>This method can be invoked from the toolbar and from the main menu.</p>
		/// <p>Images can be saved in the following formats: JPG, BMP, TIFF, GIF, EMF.</p></remarks>
		#endregion Help
		public void SaveAsImage()
		{
			if(cadImage == null) 
				return;
			if(saveImgDlg.ShowDialog() != DialogResult.OK) 
				return;
			DRect tmpRect = new DRect(0.0, 0.0, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);
			ImageFormat imgFrm = ImageFormat.Bmp;
			if(saveImgDlg.FileName == null) 
				return;			
			string tmp = saveImgDlg.FileName;	
			if(tmp.ToUpper().IndexOf(ApplicationConstants.jpgextstr) != -1)
				imgFrm = ImageFormat.Jpeg;
			if(tmp.ToUpper().IndexOf(ApplicationConstants.bmpextstr) != -1)
				imgFrm = ImageFormat.Bmp;
			if(tmp.ToUpper().IndexOf(ApplicationConstants.tiffextstr) != -1)
				imgFrm = ImageFormat.Tiff;
			if(tmp.ToUpper().IndexOf(ApplicationConstants.gifextstr) != -1)
				imgFrm = ImageFormat.Gif;
			if(tmp.ToUpper().IndexOf(ApplicationConstants.emfextstr) != -1)
				imgFrm = ImageFormat.Emf;
			if(tmp.ToUpper().IndexOf(ApplicationConstants.dxfextstr) != -1)
			{
				SaveAsDXF(saveImgDlg.FileName);
				return;
			}			
			if(! this.clipRectangle.Enabled)
			{
				rasterSizeForm.Image = this.cadImage;
				rasterSizeForm.CurrentSize = tmpRect;
				rasterSizeForm.SaveFileName = saveImgDlg.FileName;
				if(rasterSizeForm.ShowDialog() == DialogResult.OK)
					tmpRect = rasterSizeForm.SizeImage;				
			}
			int tmpNumberOfPartsInCircle = this.cadImage.NumberOfPartsInCircle;
			this.cadImage.NumberOfPartsInCircle = CADConst.SetNumberOfPartsInCurve(tmpRect);
			if(this.clipRectangle.Enabled)
			{
				tmpRect = ImageDRect;
				if((imgFrm == ImageFormat.Emf)||(imgFrm == ImageFormat.Wmf))
					cadImage.ExportToMetafile(saveImgDlg.FileName, tmpRect, this.clipRectangle.ClientRectangle);
				else
					cadImage.SaveToFile(saveImgDlg.FileName, imgFrm, tmpRect, this.clipRectangle.ClientRectangle, this.rasterSizeForm.ImagePixelFormat);
			}
			else
			{
				if((imgFrm == ImageFormat.Emf)||(imgFrm == ImageFormat.Wmf))
					cadImage.ExportToMetafile(saveImgDlg.FileName, tmpRect);
				else
					cadImage.SaveToFile(saveImgDlg.FileName, imgFrm, tmpRect,  this.rasterSizeForm.ImagePixelFormat);
			}
			this.cadImage.NumberOfPartsInCircle = tmpNumberOfPartsInCircle;					
		}

		private void miSaveFile_Click(object sender, System.EventArgs e)
		{
			SaveAsImage();	
		}

		private void miCopyAsBMP_Click(object sender, System.EventArgs e)
		{
			this.PutToClipboard();
		}

		private void PutToClipboard()
		{
			if(this.clipRectangle.Enabled)
			{
				DRect tmpRect = ImageDRect;
				cadImage.SaveImageToClipboard(tmpRect, this.clipRectangle.ClientRectangle);
			}
			else
			{
				DRect tmp = new DRect(0.0, 0.0, VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);
				cadImage.SaveImageToClipboard(tmp);
			}
		}

		#region Help
		/// <summary>
		/// Shows or hides a panel that contains the entities tree.
		/// </summary>
		#endregion Help
		public void ChangeTreeVisiblity(bool val)
		{			
			if(val)
				CADImportFace.LoadTreeNodes(trvEntity.Nodes, cadImage);
			this.spltPictBox.Visible = val;
			trvPanel.Visible = val;
			DoResize();
		}

		private void miEntitiesTree_Click(object sender, System.EventArgs e)
		{
			ChangeTreeVisiblity(! this.trvEntity.Visible);
			this.ChangeControlState();
		}

		private void miZoomIn_Click(object sender, System.EventArgs e)
		{
			DoZoomIn();
		}

		private void miZoomOut_Click(object sender, System.EventArgs e)
		{
			DoZoomOut();
		}

		private void miOptions_Click(object sender, System.EventArgs e)
		{
			this.entCreator.OptionsForm.ShowDialog();
		}

		private void miFit_Click(object sender, System.EventArgs e)
		{
			GetExtents();
		}

		#region Help
		/// <summary>
		/// Displays a CAD image in black and white colors.
		/// </summary>
		#endregion Help
		public void DoBlackColor()
		{
			if(cadImage == null) return;
			cadImage.DrawMode = CADDrawMode.Black;
			this.colorDraw = false;
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Displays a CAD image in all used colors.
		/// </summary>
		#endregion Help
		public void DoNormalColor()
		{
			if(cadImage == null) 
				return;
			cadImage.DrawMode = CADDrawMode.Normal;
			this.colorDraw = true;
			cadPictBox.Invalidate();
		}

		private void miColorDraw_Click(object sender, System.EventArgs e)
		{
			DoNormalColor();
			this.ChangeControlState();
		}

		private void miBlackDraw_Click(object sender, System.EventArgs e)
		{
			DoBlackColor();
			this.ChangeControlState();
		}

		private void miWhiteBack_Click(object sender, System.EventArgs e)
		{
			White_Click();
			this.ChangeControlState();
		}

		private void miBlackBack_Click(object sender, System.EventArgs e)
		{
			Black_Click();
			this.ChangeControlState();
		}

		private void miShowLineWeight_Click(object sender, System.EventArgs e)
		{
			ChangeShowLineWeight();
			this.ChangeControlState();
		}

		private void miArcsSplit_Click(object sender, System.EventArgs e)
		{
			UseWinEllipse();
			this.ChangeControlState();
		}

		private void miDimShow_Click(object sender, System.EventArgs e)
		{
			ChangeDimensionsVisiblity();
			this.ChangeControlState();
		}

		private void miTextsShow_Click(object sender, System.EventArgs e)
		{
			ChangeTextsVisiblity();
			this.ChangeControlState();
		}

		private void miLayersShow_Click(object sender, System.EventArgs e)
		{
			SetLayList();
			lForm.ShowDialog();
		}

		#region Help
		/// <summary>
		/// Shows or hides dimensions in the current CAD image.
		/// </summary>
		#endregion Help
		public void ChangeDimensionsVisiblity()
		{
			this.dimVisible = ! this.dimVisible;
			this.cadImage.DimensionsVisible = this.dimVisible;
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Shows or hides texts in the current CAD image.
		/// </summary>
		#endregion Help
		public void ChangeTextsVisiblity()
		{
			this.textVisible = ! this.textVisible;
			this.cadImage.TextVisible = this.textVisible;
			cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Alternately changes a boolean value that indicates if a thickness of lines is determined by the file data.
		/// </summary>
		#endregion Help
		public void ChangeShowLineWeight()
		{
			this.showLineWeight = ! this.showLineWeight;
			cadImage.IsShowLineWeight = this.showLineWeight;
			this.cadPictBox.Invalidate();
		}

		#region Help
		/// <summary>
		/// Alternately changes a boolean value indicating whether arcs and circles in the CAD image 
		/// are drawn with GDI+ drawing methods without linearization. 
		/// </summary>
		#endregion HelpBestAce
		public void UseWinEllipse()
		{
			cadImage.UseWinEllipse = ! cadImage.UseWinEllipse; 
			cadPictBox.Invalidate();
		}

		private void ChangeSelectionCond(bool val)
		{
			if(val) 
				cadImage.SelectionMode = SelectionEntityMode.Enabled;
			else
				cadImage.SelectionMode = SelectionEntityMode.Disabled;			
			this.cadImage.ClearSelectCollection();
			this.cadPictBox.Invalidate();
		}

		private void miUndo_Click(object sender, System.EventArgs e)
		{
			this.UndoChangeEntity();
		}

		private void miRedo_Click(object sender, System.EventArgs e)
		{
			this.RedoChangeEntity();
		}

		private void miDelete_Click(object sender, System.EventArgs e)
		{
			this.entCreator.Disable();
			this.RemoveEntity();
			this.stopSnap = true;
		}

		private void miPrintCustom_Click(object sender, System.EventArgs e)
		{							
			DoExtentsForPrint();
			prtForm.ShowDialog();						
		}

		private void DoExtentsForPrint()
		{
			this.cadImage.ClearSelection();
			this.cadImage.ClearMarkers();
			this.cadImage.GetExtents();			
		}

		private void miCopy_Click(object sender, System.EventArgs e)
		{
			this.cadImage.CopyEntities();
		}

		private void miPaste_Click(object sender, System.EventArgs e)
		{
			this.cadImage.PasteEntities();			
			this.cadPictBox.Invalidate();
		}

		private void miCut_Click(object sender, System.EventArgs e)
		{
			this.cadImage.CutEntities();				
			this.cadPictBox.Invalidate();
		}	
		
		private void miSerializeEntities_Click(object sender, System.EventArgs e)
		{		
			if(this.cadImage == null) 
				return;	
			string name = this.cadImage.Converter.Source;
			if(name != null)
			{
				name = Path.GetFileName(name).Replace(Path.GetExtension(name), ApplicationConstants.datextstr);			
				this.dlgSaveSerializeFile.FileName = name;
			}
			if(this.dlgSaveSerializeFile.ShowDialog() != DialogResult.OK) 
				return;		
			SortedList serializeObjects = new SortedList();		
			bool detsel = false;
			if(this.cadImage.SelectEntitiesCount != 0)
				detsel = true;
			if(detsel)
				serializeObjects.Add(ApplicationConstants.entitiesstr, this.cadImage.SelectedEntities);
			else
				serializeObjects.Add(ApplicationConstants.entitiesstr, this.cadImage.Converter.Entities);			
			serializeObjects.Add(ApplicationConstants.blocksstr, this.cadImage.Converter.Blocks);
			serializeObjects.Add(ApplicationConstants.vportsstr, this.cadImage.Converter.VPorts);
			serializeObjects.Add(ApplicationConstants.ltypesstr, this.cadImage.Converter.LTypes);
			serializeObjects.Add(ApplicationConstants.layoutsstr, this.cadImage.Converter.Layouts);
			serializeObjects.Add(ApplicationConstants.layersstr, this.cadImage.Converter.Layers);
			serializeObjects.Add(ApplicationConstants.stylesstr, this.cadImage.Converter.Styles);
			serializeObjects.Add(ApplicationConstants.xrefsstr, this.cadImage.Converter.XRefs);			
			Serialize(this.dlgSaveSerializeFile.FileName, serializeObjects);
		}

		private void miDeserializeEntities_Click(object sender, System.EventArgs e)
		{
			if(this.dlgOpenSerializeFile.ShowDialog() != DialogResult.OK)
				return;
			bool detnew = false;
			if(this.cadImage == null)
			{				
				this.cadImage = new CADImage();				
				this.cadImage.UseDoubleBuffering = false;
				this.cadImage.InitialNewImage();	
				this.cadImage.Converter.Layers.Clear();		
				this.cadImage.Converter.Layouts.Clear();	
				detnew = true;
			}		
			SortedList deserializeObjects = Deserialize(this.dlgOpenSerializeFile.FileName) as SortedList;
			if(deserializeObjects == null) 
				return;					
			LoadEntities(deserializeObjects, this.cadImage.Converter.Layers, ApplicationConstants.layersstr);
			LoadEntities(deserializeObjects, this.cadImage.Converter.Layouts, ApplicationConstants.layoutsstr);			
			LoadEntities(deserializeObjects, this.cadImage.Converter.Blocks, ApplicationConstants.blocksstr);
			LoadEntities(deserializeObjects, this.cadImage.Converter.VPorts, ApplicationConstants.vportsstr);			
			LoadEntities(deserializeObjects, this.cadImage.Converter.LTypes, ApplicationConstants.ltypesstr);			
			LoadEntities(deserializeObjects, this.cadImage.Converter.Styles, ApplicationConstants.stylesstr);						
			LoadEntities(deserializeObjects, this.cadImage.CurrentLayout.Entities, ApplicationConstants.entitiesstr);			
			this.cadImage.GetExtents();						
			if(detnew)
			{
				SetCADImageOptions();						
				SetPictureBoxPosition(Position);
				EnableButton(true);			
				SetPictureBoxLoadState(false, openFileDlg.FileName);
			}
			else
			{
				if(trvPanel.Visible == true)
				{
					trvEntity.Nodes.Clear();
					CADImportFace.LoadTreeNodes(trvEntity.Nodes, cadImage);
				}
				this.DoResize();
			}
		}
		#region Help
		#if RUSHELP
		/// <summary>
		///  
		/// </summary>
		/// <param name="fileName"> </param>
		/// <param name="sender"> </param>
		#else
		/// <summary>
		/// Serializes entities
		/// </summary>
		/// <param name="fileName">File name</param>
		/// <param name="sender">Objects to serialize</param>
		#endif		
		#endregion
		public void Serialize(string fileName, object sender)
		{
			Stream stm = File.Open(fileName, FileMode.Create, FileAccess.ReadWrite);
			BinaryFormatter bf = new BinaryFormatter();				
			bf.Serialize(stm, sender);
			stm.Close();
		}
		#region Help
#if RUSHELP
		/// <summary>
		///  
		/// </summary>
		/// <param name="fileName"> </param>		
#else
		/// <summary>
		/// Deserializes entities
		/// </summary>
		/// <param name="fileName">File name</param>
		/// <returns>Returns deserialized objects</returns>
#endif		
		#endregion
		public object Deserialize(string fileName)
		{
			Stream stm = File.Open(fileName, FileMode.Open, FileAccess.Read);
			BinaryFormatter bf = new BinaryFormatter();			
			object res = bf.Deserialize(stm);
			stm.Close();			
			return res;
		}

		private void LoadEntities(SortedList deserializeObjects, object collectAdd, string section)
		{
			CADEntity ent;		
			Object tmp;
			CADEntityCollection entities = null; 
			SortedList entitiesLst = null; 
			CADLayoutCollection layouts = null; 
			if(deserializeObjects.ContainsKey(section))
			{
				if(section == ApplicationConstants.layoutsstr)
					layouts = (CADLayoutCollection)deserializeObjects[section];						
				else
				{
					tmp = deserializeObjects[section];					
					entities = tmp as CADEntityCollection;						
					entitiesLst = tmp as SortedList;
				}
			}
			if(entities != null)			
			{
				for(int i = 0; i < entities.Count; i++)
				{
					ent = entities[i] as CADEntity;		
					if(ent == null)
						continue;							
					LoadEntity(ent, true, collectAdd);					
				}
				return;
			}
			if(entitiesLst != null)			
			{
				for(int i = 0; i < entitiesLst.Count; i++)
				{
					ent = entitiesLst.GetByIndex(i) as CADEntity;		
					if(ent == null)
						continue;							
					LoadEntity(ent, true, collectAdd);					
				}
				return;
			}
			if(layouts != null)			
			{
				for(int i = 0; i < layouts.Count; i++)
				{
					ent = layouts[i] as CADEntity;						
					if(ent == null)
						continue;							
					LoadEntity(ent, true, collectAdd);					
				}
			}
		}

		private void LoadEntity(CADEntity ent, bool detAdd, object collectAdd)
		{			
			CADLayout layout;
			CADInsert ins = ent as CADInsert;		
			if(ent is CADLeader)
				ins = (ent as CADLeader).Insert;
			if(ins != null)
			{				
				if(ins.Block != null)
				{
					for(int i = 0; i < ins.Block.Count; i++)						
						LoadEntity(ins.Block.Entities[i], false, collectAdd);											
				}				
			}			
			if(!(ent is CADImageEnt))
				ent.Loaded(this.cadImage.Converter);			
			this.cadImage.Converter.OnCreate(ent);									
			if(detAdd)				
			{				
				if(collectAdd is CADLayoutCollection)
				{	
					layout = ent as CADLayout;
					for(int i = 0; i < layout.Entities.Count; i++)						
						LoadEntity(layout.Entities[i], false, collectAdd);								
					(collectAdd as CADLayoutCollection).Add(layout);
				}
				else
					if(collectAdd is CADEntityCollection)
					(collectAdd as CADEntityCollection).Add(ent);				
			}
		}		
	}	

	internal abstract class ApplicationConstants
	{
		public static readonly string defaultstr;
		public static readonly string languagepath;
		public static readonly string loadfilestr;
		public static readonly string msgBoxDebugCaption;
		public static readonly string sepstr;
		public static readonly string notsupportedstr;
		public static readonly string notsupportedextstr;
		public static readonly string appnamestr;
		public static readonly string sepstr2;
		public static readonly string jpgextstr;
		public static readonly string bmpextstr;
		public static readonly string tiffextstr;
		public static readonly string gifextstr;
		public static readonly string emfextstr;
		public static readonly string wmfextstr;
		public static readonly string pngextstr;			
		public static readonly string dxfextstr;
		public static readonly string lngextstr;
		public static readonly string languagestr;
		public static readonly string languageIDstr;
		public static readonly string backcolorstr;
		public static readonly string blackstr;
		public static readonly string showentitystr;
		public static readonly string drawmodestr;
		public static readonly string truestr;
		public static readonly string shxpathcnstr;
		public static readonly string installstr;
		public static readonly string sepstr3;
		public static readonly string typelcstr;
		public static readonly string floatingstr;
		public static readonly string hoststr;
		public static readonly string portstr;
		public static readonly string appconst;
		public static readonly string blackstr2;
		public static readonly string whitestr;
		public static readonly string registerstr;
		public static readonly string colordrawstr;
		public static readonly string datextstr;
		public static readonly string messagestr;
		public static readonly string headstr1;
		public static readonly string headstr2;
		public static readonly string headstr3;
		public static readonly string headstr4;
		public static readonly string namestr;
		public static readonly string visiblestr;
		public static readonly string freezestr;
		public static readonly string colorstr;		
		public static readonly string entitiesstr;
		public static readonly string blocksstr;
		public static readonly string vportsstr;
		public static readonly string ltypesstr;
		public static readonly string layoutsstr;
		public static readonly string layersstr;
		public static readonly string stylesstr;
		public static readonly string xrefsstr;

		static ApplicationConstants()
		{
			defaultstr = "Default";
			languagepath = "LanguagePath";
			loadfilestr = "Loading file...";
			msgBoxDebugCaption = "Debug application message";
			sepstr = " - ";
			notsupportedstr = "not supported";
			notsupportedextstr = "Not supported in current version!";
			appnamestr = "CADImportNet Demo";
			sepstr2 = " : ";			
			jpgextstr = ".JPG";
			bmpextstr = ".BMP";
			tiffextstr = ".TIFF";
			gifextstr = ".GIF";
			emfextstr = ".EMF";
			wmfextstr = ".WMF";
			pngextstr = ".PNG";			
			dxfextstr = ".DXF";
			lngextstr = ".lng";
			languagestr = "Language";
			languageIDstr = "LanguageID";
			backcolorstr = "BackgroundColor";
			blackstr = "BLACK";
			showentitystr = "ShowEntity";
			drawmodestr = "drawMode";
			truestr = "TRUE";
			shxpathcnstr = "SHXPathCount";
			installstr = "Install";
			sepstr3 = ";";
			typelcstr = "Type_license";
			floatingstr = "floating";
			hoststr = "Host";
			portstr = "Port";
			appconst = "Application";
			blackstr2 = "Black";
			whitestr = "White";
			registerstr = "register";
			colordrawstr = "ColorDraw";
			datextstr = ".dat";
			messagestr = "If you proceed with this operation, all added objects will be lost. Continue?";
			headstr1 = "Head1";
			headstr2 = "Head2";
			headstr3 = "Head3";
			headstr4 = "Head4";
			namestr = "Name";
			visiblestr = "Visible";
			freezestr = "Freeze";
			colorstr = "Color";	
			entitiesstr = "Entities";
			blocksstr = "Blocks";
			vportsstr = "VPorts";
			ltypesstr = "LTypes";
			layoutsstr = "Layouts";
			layersstr = "Layers";
			stylesstr = "Styles";
			xrefsstr = "XRefs";
		}
	}
}
